/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <cmnfuncs.h>
#include <appconfig.h>

CAppConfig::CAppConfig(void)
{
	cfdprintf(_T("CAppConfig::CAppConfig"));

	m_fWritable = TRUE;
	m_fOpened   = FALSE;
	m_fAutoSave = FALSE;
	m_szPath    = NULL;
	m_hRootKey  = NULL;
	m_hKey      = NULL;

	InitializeCriticalSection(&m_csStr2Dw);
	InitializeCriticalSection(&m_csStr2Str);

	cfdprintf(_T("!CAppConfig::CAppConfig"));
};

CAppConfig::~CAppConfig(void)
{
	cfdprintf(_T("CAppConfig::~CAppConfig"));

	this->Close();

	DeleteCriticalSection(&m_csStr2Dw);
	DeleteCriticalSection(&m_csStr2Str);

	cfdprintf(_T("!CAppConfig::~CAppConfig"));
};

BOOL CAppConfig::SetValue(LPTSTR szName_, LPTSTR szVal_)
{
	BOOL fOk = TRUE;

	cfdprintf(_T("CAppConfig::SetValue (LPTSTR)"));

	EnterCriticalSection(&m_csStr2Str);
	fOk = this->__SetValue(szName_, szVal_);
	LeaveCriticalSection(&m_csStr2Str);

	cfdprintf(_T("!CAppConfig::SetValue (LPTSTR)"));

	return(fOk);
};

BOOL CAppConfig::SetValue(LPTSTR szName_, DWORD dwVal_)
{
	BOOL fOk = TRUE;

	cfdprintf(_T("CAppConfig::SetValue (DWORD)"));

	EnterCriticalSection(&m_csStr2Dw);
	fOk = this->__SetValue(szName_, dwVal_);
	LeaveCriticalSection(&m_csStr2Dw);

	cfdprintf(_T("!CAppConfig::SetValue (DWORD)"));

	return(fOk);
};

BOOL CAppConfig::__SetValue(LPTSTR szName_, LPTSTR szVal_)
{
	BOOL fOk = TRUE;

	cfdprintf(_T("CAppConfig::__SetValue (LPTSTR)"));

	assert(szVal_ != NULL);

	EnterCriticalSection(&m_csStr2Str);
	m_mapStr2Str[szName_] = szVal_;

	if(m_fAutoSave)
		fOk = SaveValue(szName_, szVal_);

	LeaveCriticalSection(&m_csStr2Str);

	cfdprintf(_T("!CAppConfig::__SetValue (LPTSTR)"));

	return(fOk);
};

BOOL CAppConfig::__SetValue(LPTSTR szName_, DWORD dwVal_)
{
	BOOL fOk = TRUE;

	cfdprintf(_T("CAppConfig::__SetValue (DWORD)"));

	EnterCriticalSection(&m_csStr2Dw);
	m_mapStr2Dw[szName_] = dwVal_;

	if(m_fAutoSave)
		fOk = SaveValue(szName_, dwVal_);

	LeaveCriticalSection(&m_csStr2Dw);

	cfdprintf(_T("!CAppConfig::__SetValue (DWORD)"));

	return(fOk);
};

BOOL CAppConfig::GetValue(LPTSTR szName_, LPTSTR* pszOutVal_, LPTSTR szDef_)
{
	BOOL fOk = FALSE;
	STR2STRIT it;
	LPTSTR szOutVal = NULL;

	cfdprintf(_T("CAppConfig::GetValue (LPTSTR)"));

	EnterCriticalSection(&m_csStr2Str);

	it = m_mapStr2Str.find(szName_);
	if(it != m_mapStr2Str.end()) // If we have data in the local cache, return it.
	{
		LPTSTR szVal;

		cfdprintf(_T("CAppConfig::GetValue (LPTSTR): Getting from local cache."));

		szVal = (LPTSTR)it->second.c_str();
		szOutVal = new TCHAR[_tcslen(szVal)+1];
		_tcscpy(szOutVal, szVal);
		fOk = TRUE;
	}
	else // If there is no data in the local cache, try to read it from system registry.
	{
		LONG lRet;

		cfdprintf(_T("CAppConfig::GetValue (LPTSTR): Getting from system registry."));

		assert(m_hKey != NULL);

		lRet = cfRegReadSTRVal(m_hKey, szName_, &szOutVal, szDef_);

		cfdprintf(_T("CAppConfig::GetValue (LPTSTR): Name: %s; Val: %s; lRet: %d"), szName_, szOutVal, lRet);

		fOk = (lRet == ERROR_SUCCESS ? TRUE : FALSE);

		if(szOutVal != NULL) // Save value in the local cache.
			__SetValue(szName_, szOutVal);
	};

	if(pszOutVal_ != NULL)
		*pszOutVal_ = szOutVal;
	else
		delete[] szOutVal;

	LeaveCriticalSection(&m_csStr2Str);

	cfdprintf(_T("!CAppConfig::GetValue (LPTSTR)"));

	return(fOk);
};

BOOL CAppConfig::GetValue(LPTSTR szName_, LPDWORD pdwOutVal_, DWORD dwDef_)
{
	BOOL fOk = FALSE;
	STR2DWIT it;
	DWORD dwOutVal = 0;

	cfdprintf(_T("CAppConfig::GetValue (DWORD)"));

	EnterCriticalSection(&m_csStr2Dw);

	it = m_mapStr2Dw.find(szName_);
	if(it != m_mapStr2Dw.end()) // If we have data in the local cache, return it.
	{
		cfdprintf(_T("CAppConfig::GetValue (DWORD): Getting from local cache."));
		dwOutVal = it->second;
	}
	else // If there is no data in the local cache, try to read it from system registry.
	{
		LONG lRet;

		cfdprintf(_T("CAppConfig::GetValue (DWORD): Getting from system registry."));

		assert(m_hKey != NULL);

		lRet = cfRegReadDWVal(m_hKey, szName_, &dwOutVal, dwDef_);

		fOk = (lRet == ERROR_SUCCESS ? TRUE : FALSE);

		// Save value in the local cache.
		__SetValue(szName_, dwOutVal);
	};

	if(pdwOutVal_ != NULL)
		*pdwOutVal_ = dwOutVal;

	LeaveCriticalSection(&m_csStr2Dw);

	cfdprintf(_T("!CAppConfig::GetValue (DWORD)"));

	return(fOk);
};

BOOL CAppConfig::Save(VOID)
{
	BOOL fOk = FALSE;

	cfdprintf(_T("CAppConfig::Save"));

	EnterCriticalSection(&m_csStr2Str);
	EnterCriticalSection(&m_csStr2Dw);

	LeaveCriticalSection(&m_csStr2Str);
	LeaveCriticalSection(&m_csStr2Dw);

	cfdprintf(_T("!CAppConfig::Save"));

	return(fOk);
};

BOOL CAppConfig::Open(HKEY hKey_, LPTSTR szPath_)
{
	LONG lRet;

	cfdprintf(_T("CAppConfig::Open"));

	assert(m_szPath == NULL);
	assert(m_hRootKey == NULL);
	assert(m_hKey == NULL);

	m_szPath = new TCHAR[_tcslen(szPath_)+1];
	_tcscpy(m_szPath, szPath_);
	m_fWritable = TRUE;
	m_hRootKey = hKey_;

next_try:
	lRet = RegCreateKeyEx(m_hRootKey, m_szPath, 0, NULL, REG_OPTION_NON_VOLATILE, KEY_READ|(m_fWritable ? KEY_WRITE : 0), NULL, &m_hKey, NULL);
	if(lRet != ERROR_SUCCESS)
	{
		cfdprintf(_T("CAppConfig::Open: Call to RegCreateKeyEx failed (Ret: %d; RW: %d)"), lRet, m_fWritable);
		if(lRet == ERROR_ACCESS_DENIED && m_fWritable)
		{
			cfdprintf(_T("CAppConfig::Open: Call to RegCreateKeyEx failed with access denied for RW operation, let's try to open RO."), lRet);
			m_fWritable = FALSE;
			goto next_try;
		};
	};

	m_fOpened = (lRet == ERROR_SUCCESS ? TRUE : FALSE);

	cfdprintf(_T("!CAppConfig::Open (lRet: %d)"), lRet);

	return(m_fOpened);
};

VOID CAppConfig::Close(VOID) 
{
	cfdprintf(_T("CAppConfig::Close"));

	RegCloseKey(m_hKey);

	if(m_szPath) delete[] m_szPath;

	m_szPath   = NULL;
	m_hRootKey = NULL;
	m_hKey     = NULL;
	m_szPath   = NULL;

	cfdprintf(_T("!CAppConfig::Close"));
};

BOOL CAppConfig::SaveValue(LPTSTR szName_, LPTSTR szVal_)
{
	LONG lRet;

	cfdprintf(_T("CAppConfig::SaveValue (LPTSTR) Name: %s; Value: %s"), szName_, szVal_);

	lRet = cfRegSaveSTRVal(m_hKey, szName_, szVal_);

	cfdprintf(_T("!CAppConfig::SaveValue (LPTSTR) Name: %s"), szName_);

	return(lRet == ERROR_SUCCESS ? TRUE : FALSE);
};

BOOL CAppConfig::SaveValue(LPTSTR szName_, DWORD dwVal_)
{
	LONG lRet;

	cfdprintf(_T("CAppConfig::SaveValue (DWORD) Name: %s; Value: %d"), szName_, dwVal_);

	lRet = cfRegSaveDWVal(m_hKey, szName_, dwVal_);

	cfdprintf(_T("!CAppConfig::SaveValue (DWORD) Name: %s"), szName_);

	return(lRet == ERROR_SUCCESS ? TRUE : FALSE);
};
