/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <bblnc.h>
#include <bbobj.h>
#include <process.h>
#include <atlbase.h>
#include <atlconv.h>
#include <objbase.h>
#include <shellapi.h>
#include <notifyicon.h>
#include <window.h>
#include <botiknadminobj_h.h>
#include <notifywindow.h>
#include <mainwindow.h>

CMainWindow* MainWindow = NULL;

CMainWindow::CMainWindow(void) : CWindow()
{
	cfdprintf(_T("CMainWindow::CMainWindow"));

	m_hWnd        = NULL;
	m_fChanged    = FALSE;
	m_fTimerArmed = FALSE;
	m_fShowAlarm  = TRUE;

	cfdprintf(_T("!CMainWindow::CMainWindow"));
};

CMainWindow::~CMainWindow(void)
{
	cfdprintf(_T("CMainWindow::~CMainWindow"));
	cfdprintf(_T("!CMainWindow::~CMainWindow"));
};

BOOL CMainWindow::CreateObject()
{
	assert(MainWindow == NULL);
	MainWindow = new CMainWindow();
	if(!MainWindow)
		return(FALSE);
	return(TRUE);
};

BOOL CMainWindow::Create(HWND hWndPrn_)
{
	BOOL fOk = TRUE;

	cfdprintf(_T("CMainWindow::Create"));

	assert(m_hWnd == NULL);

	m_hWnd = ::CreateDialog(g_hInst, MAKEINTRESOURCE(IDD_CONFIG_DLG), NULL, (DLGPROC)_WndProc);

	if(!m_hWnd)
	{
		cfdprintf(_T("CMainWindow::Create: CreateDialog failed"));
		fOk = FALSE;
	};

	cfdprintf(_T("!CMainWindow::Create"));

	return(fOk);
};

VOID CMainWindow::UpdateGUI(HWND hWnd_)
{
	LPTSTR szVal = NULL;
	DWORD  dwVal = 0;

	cfdprintf(_T("CMainWindow::UpdateGUI"));

	g_pAppConfigUser->GetValue(BBLNC_ACCOUNTNAME, &szVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set Account Name: %s"), szVal);
	::SetDlgItemText(hWnd_, IDC_EDIT_ACCOUNT, szVal);
	delete[] szVal;

	g_pAppConfigUser->GetValue(BBLNC_USER, &szVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set User: %s"), szVal);
	::SetDlgItemText(hWnd_, IDC_EDIT_USER, szVal);
	delete[] szVal;

	g_pAppConfigUser->GetValue(BBLNC_PASSWD, &szVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set Password: %s"), szVal);
	::SetDlgItemText(hWnd_, IDC_EDIT_PASSWD, szVal);
	delete[] szVal;

	g_pAppConfigUser->GetValue(BBLNC_CRITICALACCOUNTSTATE, &dwVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set Low account val: %d"), dwVal);
	::SetDlgItemInt(hWnd_, IDC_EDIT_LOW_ACCOUNT_VAL, dwVal, FALSE);

	g_pAppConfigUser->GetValue(BBLNC_INFOWINDOWTIMEOUT, &dwVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set InfoWindow Timeout: %d"), dwVal);
	::SetDlgItemInt(hWnd_, IDC_EDIT_SHOW_DURATION, dwVal, FALSE);

	g_pAppConfigUser->GetValue(BBLNC_UPDATETIMEOUT, &dwVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set update timeout: %d"), dwVal);
	::SetDlgItemInt(hWnd_, IDC_EDIT_UPDATETIMEOUT, dwVal, FALSE);

	g_pAppConfigUser->GetValue(BBLNC_AUTOUPDATE, &dwVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set Autoupdate: %d"), dwVal);
	SendDlgItemMessage(hWnd_, IDC_CHECK_AUTOUPDATE, BM_SETCHECK, dwVal, 0);

	cfdprintf(_T("CMainWindow::UpdateGUI: Set Autoupdate: %d"), g_fShowErrors);
	SendDlgItemMessage(hWnd_, IDC_CHECK_SHOWERRORS, BM_SETCHECK, g_fShowErrors, 0);

	// Show info window type.
	g_pAppConfigUser->GetValue(BBLNC_SHOWINFOWINDOW, &dwVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set Show InfoWindow Type: %d"), dwVal);

	if(dwVal == 0)
		SendDlgItemMessage(hWnd_, IDC_RADIO_USER_DEMAND, BM_CLICK, 1, 0);
	else
		SendDlgItemMessage(hWnd_, dwVal, BM_CLICK, 1, 0);

	// Set slider pos.
	g_pAppConfigUser->GetValue(BBLNC_INFOWINDOWTRANSP, &dwVal, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Set transparent slider pos: %d"), dwVal);

	SendDlgItemMessage(hWnd_, IDC_SLIDER_TRANSPARENT, TBM_SETLINESIZE, 0, 1);
	SendDlgItemMessage(hWnd_, IDC_SLIDER_TRANSPARENT, TBM_SETRANGE, TRUE, MAKELONG(10, 255));
	SendDlgItemMessage(hWnd_, IDC_SLIDER_TRANSPARENT, TBM_SETPOS, TRUE, dwVal);

	if(!pSetLayeredWindowAttributes)
		cfEnableDlgItem(hWnd_, IDC_SLIDER_TRANSPARENT, FALSE);
    else
        pSetLayeredWindowAttributes(NotifyWindow->GetHWnd(), 0, (BYTE)dwVal, LWA_ALPHA);

	// Check if programm autostarts at user logon.
	dwVal = cfRegReadSTRVal(g_hKeyAS, BBLNC_REG_AUTOSTART_VAL, NULL, NULL);
	cfdprintf(_T("CMainWindow::UpdateGUI: Autostart: %d"), !dwVal);
	SendDlgItemMessage(hWnd_, IDC_CHECK_AUTOSTART,  BM_SETCHECK, !dwVal, 0);

	cfdprintf(_T("!CMainWindow::UpdateGUI"));
};

BOOL CMainWindow::ValidateNewConfiguration()
{
	DWORD dwVal;

	dwVal = cfGetDlgItemTextLength(m_hWnd, IDC_EDIT_ACCOUNT);
	if(dwVal <= 0)
	{
		::MessageBox(m_hWnd, _T(":       "), BBLNC_MSGBOX_CAPTION, MB_OK|MB_ICONHAND);
		cfSetDlgItemFocus(m_hWnd, IDC_EDIT_ACCOUNT);
		return(FALSE);
	};

	dwVal = cfGetDlgItemTextLength(m_hWnd, IDC_EDIT_USER);
	if(dwVal <= 0)
	{
		::MessageBox(m_hWnd, _T(":      "), 
					 BBLNC_MSGBOX_CAPTION, MB_OK|MB_ICONHAND);
		cfSetDlgItemFocus(m_hWnd, IDC_EDIT_USER);
		return(FALSE);
	};

	dwVal = cfGetDlgItemTextLength(m_hWnd, IDC_EDIT_PASSWD);
	if(dwVal <= 0)
	{
		::MessageBox(m_hWnd, _T(":      "), 
					 BBLNC_MSGBOX_CAPTION, MB_OK|MB_ICONHAND);
		cfSetDlgItemFocus(m_hWnd, IDC_EDIT_PASSWD);
		return(FALSE);
	};

	dwVal = GetDlgItemInt(m_hWnd, IDC_EDIT_UPDATETIMEOUT, NULL, FALSE);
	if(dwVal < 5)
	{
		::MessageBox(m_hWnd, _T(":        5 "), 
					 BBLNC_MSGBOX_CAPTION, MB_OK|MB_ICONHAND);
		cfSetDlgItemFocus(m_hWnd, IDC_EDIT_UPDATETIMEOUT);
		return(FALSE);
	};

	return(TRUE);
};

BOOL CMainWindow::UpdateSettings(VOID)
{
	LONG   lLen;
	LPTSTR szBuf;
	DWORD  dwVal;
	DWORD  dwTmp;
	TCHAR  szPath[MAX_PATH+1];

	lLen = cfGetDlgItemTextLength(m_hWnd, IDC_EDIT_ACCOUNT);
	szBuf = new TCHAR[lLen+2]; *szBuf = 0;
	GetDlgItemText(m_hWnd, IDC_EDIT_ACCOUNT, szBuf, lLen+1);
	g_pAppConfigUser->SetValue(BBLNC_ACCOUNTNAME, szBuf);
	delete[] szBuf;

	lLen = cfGetDlgItemTextLength(m_hWnd, IDC_EDIT_USER);
	szBuf = new TCHAR[lLen+2]; *szBuf = 0;
	GetDlgItemText(m_hWnd, IDC_EDIT_USER, szBuf, lLen+1);
	g_pAppConfigUser->SetValue(BBLNC_USER, szBuf);
	delete[] szBuf;

	lLen = cfGetDlgItemTextLength(m_hWnd, IDC_EDIT_PASSWD);
	szBuf = new TCHAR[lLen+2]; *szBuf = 0;
	GetDlgItemText(m_hWnd, IDC_EDIT_PASSWD, szBuf, lLen+1);
	g_pAppConfigUser->SetValue(BBLNC_PASSWD, szBuf);
	delete[] szBuf;

	// Set or delete startup path.
	dwVal = (DWORD)SendDlgItemMessage(m_hWnd, IDC_CHECK_AUTOSTART, BM_GETCHECK, 0, 0);
	if(dwVal)
	{
		GetModuleFileName(g_hInst, szPath, MAX_PATH);
		cfRegSaveSTRVal(g_hKeyAS, BBLNC_REG_AUTOSTART_VAL, szPath);
	}
	else
		RegDeleteValue(g_hKeyAS, BBLNC_REG_AUTOSTART_VAL);

	dwVal = GetDlgItemInt(m_hWnd, IDC_EDIT_UPDATETIMEOUT, NULL, FALSE);
	g_pAppConfigUser->SetValue(BBLNC_UPDATETIMEOUT, dwVal);

	dwVal = (DWORD)SendDlgItemMessage(m_hWnd, IDC_CHECK_AUTOUPDATE, BM_GETCHECK, 0, 0);
	g_pAppConfigUser->SetValue(BBLNC_AUTOUPDATE, dwVal);
	if(dwVal)
		StartAutoUpdate();
	else
		StopAutoUpdate();

	g_fShowErrors = (DWORD)SendDlgItemMessage(m_hWnd, IDC_CHECK_SHOWERRORS, BM_GETCHECK, 0, 0);
	g_pAppConfigUser->SetValue(BBLNC_SHOWERRORS, g_fShowErrors);

	dwVal = GetDlgItemInt(m_hWnd, IDC_EDIT_LOW_ACCOUNT_VAL, NULL, FALSE);
	g_pAppConfigUser->GetValue(BBLNC_CRITICALACCOUNTSTATE, &dwTmp, NULL);
	g_pAppConfigUser->SetValue(BBLNC_CRITICALACCOUNTSTATE, dwVal);

	if(dwTmp != dwVal)
		m_fShowAlarm = TRUE;

	dwVal = GetDlgItemInt(m_hWnd, IDC_EDIT_SHOW_DURATION, NULL, FALSE);
	g_pAppConfigUser->SetValue(BBLNC_INFOWINDOWTIMEOUT, dwVal);

	if(SendDlgItemMessage(m_hWnd, IDC_RADIO_USER_DEMAND, BM_GETCHECK, 0, 0))
		dwVal = IDC_RADIO_USER_DEMAND;
	else if(SendDlgItemMessage(m_hWnd, IDC_RADIO_STATUS_CHANGED, BM_GETCHECK, 0, 0))
		dwVal = IDC_RADIO_STATUS_CHANGED;
	else if(SendDlgItemMessage(m_hWnd, IDC_RADIO_STATUS_GET_VALUE, BM_GETCHECK, 0, 0))
		dwVal = IDC_RADIO_STATUS_GET_VALUE;

	g_pAppConfigUser->SetValue(BBLNC_SHOWINFOWINDOW, dwVal);

	dwVal = (DWORD)SendDlgItemMessage(m_hWnd, IDC_SLIDER_TRANSPARENT, TBM_GETPOS, 0, 0);
	g_pAppConfigUser->SetValue(BBLNC_INFOWINDOWTRANSP, dwVal);

	return(TRUE);
};

VOID CMainWindow::StartTrayAnimation()
{
	::SetTimer(m_hWnd, TRAY_ANIMATION_TIMER_ID, 1000, (TIMERPROC)NULL);
};

VOID CMainWindow::StopTrayAnimation()
{
	::KillTimer(m_hWnd, TRAY_ANIMATION_TIMER_ID);
};

VOID CMainWindow::StartAutoUpdate()
{
	DWORD dwVal;

	g_pAppConfigUser->GetValue(BBLNC_AUTOUPDATE, &dwVal, NULL);

	if(dwVal)
	{
		g_pAppConfigUser->GetValue(BBLNC_UPDATETIMEOUT, &dwVal, NULL);
		::SetTimer(m_hWnd, UPDATE_TIMEOUT_TIMER_ID, dwVal*60*1000, (TIMERPROC)NULL);
	};
};

VOID CMainWindow::StopAutoUpdate()
{
	::KillTimer(m_hWnd, UPDATE_TIMEOUT_TIMER_ID);
};

VOID CMainWindow::StartUpdating()
{
	cfdprintf(_T("CMainWindow::StartUpdating"));

	SetCurrentState(BBLNC_CURRENT_STATE_DATA_UPDATING);
	::SetEvent(m_hEvent);

	cfdprintf(_T("!CMainWindow::StartUpdating"));
};

VOID CMainWindow::ShowCurrentStateIcon()
{
	if(g_nCurState == BBLNC_CURRENT_STATE_DATA_NA)
	{
		StopTrayAnimation();
		m_pNIcon->ShowIcon(1);
	} 
	else if(g_nCurState == BBLNC_CURRENT_STATE_DATA_UPDATING)
	{
		m_pNIcon->SetRotatingRange(2, 3);
		StartTrayAnimation();
	} 
	else if(g_nCurState == BBLNC_CURRENT_STATE_OVER_VALUE)
	{
		m_pNIcon->SetRotatingRange(4, 5);
		StartTrayAnimation();
	} 
	else if(g_nCurState == BBLNC_CURRENT_STATE_UPDATE_FAILED)
	{
		StopTrayAnimation();
		m_pNIcon->ShowIcon(6);
	}
	else
	{
		StopTrayAnimation();
		m_pNIcon->ShowIcon(0);
	};
};

VOID CMainWindow::SetCurrentState(INT nCurState_)
{
	g_nCurState = nCurState_;
	ShowCurrentStateIcon();
};

BOOL CMainWindow::Startup()
{
	cfdprintf(_T("CMainWindow::Startup"));

	if(g_fFirstStart)
	{
		cfdprintf(_T("CMainWindow::Startup: First start. Updating thread will be executed late, after program configuration."));
		return(0);
	};

	m_hEvent = CreateEvent(NULL, FALSE, FALSE, NULL);
	assert(m_hEvent != NULL);
	if(m_hEvent == NULL)
	{
		cfdprintf(_T("CMainWindow::Startup: CreateEvent failed (%d)"), GetLastError());
		return(0);
	};

	// Start updating thread.
	unsigned int thrdid;
	_beginthreadex(NULL, 0, _UpdateDataThread, NULL, 0, &thrdid);

	StartUpdating();

	cfdprintf(_T("!CMainWindow::Startup"));

	return(TRUE);
};

///////////////////////////////////////////////////////////////////////////////
// WM event handlers
///////////////////////////////////////////////////////////////////////////////

LRESULT CMainWindow::OnInitDialog(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CMainWindow::OnInitDialog"));

	WM_TASKBARCREATED = ::RegisterWindowMessage(_T("TaskbarCreated"));

	this->UpdateGUI(hWnd_);

	m_pNIcon = new CNotifyIcon();
	m_pNIcon->Create(g_hInst, hWnd_, BBLNC_NOTIFYICON);
	m_pNIcon->AddIcon(IDI_BBLNC_TRAY, _T("Botik::Balance"));
	m_pNIcon->AddIcon(IDI_BBLNC_TRAY_NA, _T("Botik::Balance\n ..."));
	m_pNIcon->AddIcon(IDI_BBLNC_TRAY_UPD, _T("Botik::Balance\n ..."));
	m_pNIcon->AddIcon(IDI_BBLNC_TRAY, _T("Botik::Balance\n ..."));
	m_pNIcon->AddIcon(IDI_BBLNC_TRAY_OVER, _T("Botik::Balance"));
	m_pNIcon->AddIcon(IDI_BBLNC_TRAY, _T("Botik::Balance"));
	m_pNIcon->AddIcon(IDI_BBLNC_TRAY_ERR, _T("Botik::Balance\n ..."));

	ShowCurrentStateIcon();

	cfdprintf(_T("!CMainWindow::OnInitDialog"));

	return(TRUE);
};

LRESULT CMainWindow::OnDestroy(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CMainWindow::OnDestroy"));

	NotifyWindow->Destroy();
	m_pNIcon->HideIcon();
	m_pNIcon->Destroy();
	PostQuitMessage(0);

	cfdprintf(_T("!CMainWindow::OnDestroy"));

	return(TRUE);
};

LRESULT CMainWindow::OnCommand(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CMainWindow::OnCommand"));

	LRESULT lRes = FALSE;
	INT wmId     = LOWORD(wParam_);
	INT wmEvent  = HIWORD(wParam_);
	BOOL fState;

	switch(wmEvent)
	{
		case BN_CLICKED:
			fState = (BOOL)SendDlgItemMessage(hWnd_, IDC_RADIO_STATUS_GET_VALUE, BM_GETCHECK, 0, 0);
			cfEnableDlgItem(hWnd_, IDC_EDIT_LOW_ACCOUNT_VAL, fState);

			fState = (BOOL)SendDlgItemMessage(hWnd_, IDC_CHECK_AUTOUPDATE, BM_GETCHECK, 0, 0);
			cfEnableDlgItem(hWnd_, IDC_EDIT_UPDATETIMEOUT, fState);
			cfEnableDlgItem(hWnd_, IDC_STATIC_AUTOUPDATE_0, fState);
			cfEnableDlgItem(hWnd_, IDC_STATIC_AUTOUPDATE_1, fState);
		case EN_CHANGE:
		break;
	};

	switch(wmId)
	{
		// Menu commands...
		case IDM_CONFIG:
			this->Show(SW_SHOW);
			this->Show(SW_RESTORE);
		break;

		case IDM_EXIT:
			this->Destroy();
			lRes = TRUE;
		break;

		case IDM_ABOUT:
			DialogBox(g_hInst, MAKEINTRESOURCE(IDD_ABOUT_DLG), hWnd_, (DLGPROC)_WndProcAboutDlg);
			lRes = TRUE;
		break;

		case IDM_SHOW_LICENSE:
			DialogBox(g_hInst, MAKEINTRESOURCE(IDD_LICENSE_DLG), hWnd_, (DLGPROC)_WndProcLicenseDlg);
			lRes = TRUE;
		break;

		case IDM_SHOW_HELP:
		break;

		case IDM_OPENNADMIN:
			{
				LPTSTR szAccount = NULL;
				LPTSTR szUser = NULL;
				LPTSTR szPass = NULL;
				LPTSTR szUrl = NULL;

				g_pAppConfigUser->GetValue(BBLNC_ACCOUNTNAME, &szAccount, NULL);
				g_pAppConfigUser->GetValue(BBLNC_USER, &szUser, NULL);
				g_pAppConfigUser->GetValue(BBLNC_PASSWD, &szPass, NULL);

//				szUrl = cfsprintf(BBLNC_NADMIN_LINK_SITE, szAccount, szUser, szPass);
				szUrl = cfsprintf(BBLNC_NADMIN_LINK_POST, szAccount, szUser, szPass);
//				ShellExecute(HWND_DESKTOP, _T("open"), szUrl, NULL, NULL, SW_SHOWNORMAL);
				ShellExecute(HWND_DESKTOP, _T("open"), BBLNC_NADMIN_LINK_SITE, szUrl, NULL, SW_SHOWNORMAL);
				lRes = TRUE;

				delete[] szAccount;
				delete[] szUser;
				delete[] szPass;
				delete[] szUrl;
			}
		break;

		case IDM_WEB_SITE:
			ShellExecute(HWND_DESKTOP, _T("open"), BBLNC_DEV_LINK_SITE, NULL, NULL, SW_SHOWNORMAL);
			lRes = TRUE;
		break;

		case IDM_UPDATE:
			this->StartUpdating();
			lRes = TRUE;
		break;

		case IDM_PUTINCLIPBOARD:
			lRes = TRUE;
		break;

		case IDM_SHOWINFOWINDOW:
			NotifyWindow->PopUp();
			lRes = TRUE;
		break;

		// Config dialog commands...
		case IDOK:
			if(g_fFirstStart) // Check for first start.
			{
				g_pAppConfigUser->SetValue(BBLNC_FIRST_START, (DWORD)FALSE);
				g_fFirstStart = FALSE;
				MainWindow->Startup();
			};

			if(!ValidateNewConfiguration())
			{
				lRes = TRUE;
				break;
			};

			if(!this->UpdateSettings())
				lRes = TRUE;

		case IDCANCEL:
			this->Show(SW_MINIMIZE);
			this->Show(SW_HIDE);
			this->UpdateGUI(hWnd_);
			lRes = TRUE;
		break;
	};

	cfdprintf(_T("!CMainWindow::OnCommand (lRes = %d)"), lRes);

	return(lRes);
};

LRESULT CMainWindow::OnNotify(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CMainWindow::OnNotify"));

    if(wParam_ == IDC_SLIDER_TRANSPARENT)
    {
        LPNMHDR lpnmh = (LPNMHDR)lParam_;

        if(lpnmh->code == NM_CUSTOMDRAW && pSetLayeredWindowAttributes)
        {
            // This notification is sent each time we sliding a trackbar to redraw it, 
            // so let's use it to track position during user moves a thumb and update
            // window opacity accordingly.
            DWORD dwVal = (DWORD)SendDlgItemMessage(hWnd_, IDC_SLIDER_TRANSPARENT, TBM_GETPOS, 0, 0);
            pSetLayeredWindowAttributes(NotifyWindow->GetHWnd(), 0, (BYTE)dwVal, LWA_ALPHA);
        };
    };

	cfdprintf(_T("!CMainWindow::OnNotify"));

	return(TRUE);
};

LRESULT CMainWindow::OnUserNotifyIcon(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	DWORD        dwMicePos;
	POINT        pt;
	HMENU        hMenu, hSubMenu;
	MENUITEMINFO mii;

//	cfdprintf("CMainWindow::OnUserNotifyIcon");

	dwMicePos = GetCursorPos(&pt);

	switch(lParam_)
	{
		case WM_LBUTTONUP:
			if(!m_fTimerArmed) // This deleay is needed to descern 
			{                  // if user press one time or double time.
				UINT uiDblTime;

				uiDblTime = GetDoubleClickTime();

				m_fTimerArmed = TRUE;
				::SetTimer(hWnd_, SHOWINFOWND_DELEY_TIMER_ID, uiDblTime, (TIMERPROC)NULL);
			}
			else
				m_fTimerArmed = FALSE;
			break;

		case WM_LBUTTONDBLCLK:
			::KillTimer(m_hWnd, SHOWINFOWND_DELEY_TIMER_ID);
			this->Show(SW_SHOW);
			this->Show(SW_RESTORE);
			break;

		case WM_MBUTTONUP:
			break;

		case WM_RBUTTONUP:
			hMenu = LoadMenu(g_hInst, MAKEINTRESOURCE(IDC_ICON_MENU));
			hSubMenu = GetSubMenu(hMenu, 0);

			mii.cbSize = sizeof(MENUITEMINFO);
			mii.fMask  = MIIM_STATE;
			mii.fState = MFS_DEFAULT;
			SetMenuItemInfo(hSubMenu, 0, TRUE, &mii);

			mii.fState = g_fUpdateThrdReady ? MFS_ENABLED : MFS_DISABLED;
			SetMenuItemInfo(hSubMenu, 2, TRUE, &mii);


			// NB. We _must_ set our window to foreground before call
			// TrackPopupMenu(Ex), otherwise, the menu will not disappear
			// when user clicks outside of the menu.
			SetForegroundWindow(hWnd_);

			TrackPopupMenuEx(hSubMenu, TPM_LEFTALIGN|TPM_LEFTBUTTON|TPM_VERTICAL, pt.x, pt.y, hWnd_, NULL);

			DestroyMenu(hMenu);
			break;
	};

//	cfdprintf("!CMainWindow::OnUserNotifyIcon");

	return(TRUE);
};

LRESULT CMainWindow::OnTimer(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	if(wParam_ == TRAY_ANIMATION_TIMER_ID)
		m_pNIcon->OnTimer(hWnd_, wParam_, lParam_);

	if(wParam_ == UPDATE_TIMEOUT_TIMER_ID)
		StartUpdating();

	if(wParam_ == SHOWINFOWND_DELEY_TIMER_ID)
	{
		::KillTimer(m_hWnd, SHOWINFOWND_DELEY_TIMER_ID);
		m_fTimerArmed = FALSE;
		NotifyWindow->PopUp();
	};

	return(TRUE);
};

///////////////////////////////////////////////////////////////////////////////
// Window procedures
///////////////////////////////////////////////////////////////////////////////

LRESULT CMainWindow::WndProc(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	LRESULT lRes = 0;

//	cfdprintf(_T("CMainWindow::WndProc"));

	switch(uMsg_)
	{
		case WM_INITDIALOG:
			lRes = OnInitDialog(hWnd_, wParam_, lParam_);
			break;

		case WM_DESTROY:
			lRes = OnDestroy(hWnd_, wParam_, lParam_);
			break;

		case WM_COMMAND:
			lRes = OnCommand(hWnd_, wParam_, lParam_);
			break;

		case WM_NOTIFY:
			lRes = OnNotify(hWnd_, wParam_, lParam_);
			break;

		case WM_USER_NOTIFYICON:
			OnUserNotifyIcon(hWnd_, wParam_, lParam_);
			break;

		case WM_TIMER:
			lRes = OnTimer(hWnd_, wParam_, lParam_);
			break;

		default:
			if(uMsg_ == WM_TASKBARCREATED) // To restore notification icon after explorer.exe crashed.
			{
				m_pNIcon->OnTaskbarCreated(hWnd_, wParam_, lParam_);
			};
			break;
	};

//	cfdprintf(_T("!CMainWindow::WndProc (lRes = %d)"), lRes);

	return(lRes);
};

LRESULT CMainWindow::_WndProc(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	assert(MainWindow != NULL);
	return(MainWindow->WndProc(hWnd_, uMsg_, wParam_, lParam_));
};

LRESULT CMainWindow::WndProcAboutDlg(HWND hDlg_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	static HCURSOR hHandCur = LoadCursor(g_hInst, MAKEINTRESOURCE(IDC_MY_HAND));
	static HCURSOR hOldCur = NULL;

	switch (uMsg_)
	{
	case WM_INITDIALOG:
		{
			TCHAR szText[MAX_LOADSTRING+1];

			// Version text.
			LoadString(g_hInst, IDS_ABOUT_TITLE, szText, MAX_LOADSTRING);
			SetDlgItemText(hDlg_, IDC_ABOUT_TITLE, szText);			

			// Credits.
			HRSRC hrc;
			HGLOBAL hgl;
			LPTSTR szStr, szPtr;
			DWORD dwSize;

			hrc = FindResource((HMODULE)g_hInst, MAKEINTRESOURCE(IDR_CREDITS), RT_HTML);
			if(hrc == NULL)
				return(FALSE);

			dwSize = SizeofResource((HMODULE)g_hInst, hrc);

			hgl = LoadResource((HMODULE)g_hInst, hrc);
			if(hgl == NULL)
				return(FALSE);

			szPtr = (LPTSTR)LockResource(hgl);
			if(szPtr == NULL)
				return(FALSE);

			szStr = new TCHAR[dwSize + 1];
			memcpy(szStr, szPtr, dwSize);
			szStr[dwSize] = 0;

			SetDlgItemText(hDlg_, IDC_THANKS_TEXT, szStr);

			delete[] szStr;
			FreeResource(hgl);

			// Setup email and URL look and feel like links, color of the lik text is changing in the 
			// WM_CTLCOLORSTATIC message.
			LOGFONT lf;

			// Get default GUI font information.
			GetObject(GetStockObject(DEFAULT_GUI_FONT), sizeof(LOGFONT), &lf);

			// Change some params.
			lf.lfUnderline = TRUE;

			HFONT hFont = CreateFont(lf.lfHeight, lf.lfWidth, lf.lfEscapement, 
									lf.lfOrientation, lf.lfWeight, lf.lfItalic, 
									lf.lfUnderline, lf.lfStrikeOut, lf.lfCharSet, 
									lf.lfOutPrecision, lf.lfClipPrecision, lf.lfQuality, 
									lf.lfPitchAndFamily, lf.lfFaceName);

			SendDlgItemMessage(hDlg_, IDC_EMAIL, WM_SETFONT, (WPARAM)hFont, TRUE);
			SendDlgItemMessage(hDlg_, IDC_URL, WM_SETFONT, (WPARAM)hFont, TRUE);

			//HDC hdc = GetWindowDC(GetDlgItem(hDlg_, IDC_EMAIL));
			//SetTextColor(hdc, RGB(0,0,255));
			//SetBkMode(hdc, TRANSPARENT);
		};
		return TRUE;

	case WM_CTLCOLORSTATIC:
		{
			//HWND hWnd = (HWND)lParam_;
			//HDC hdc = (HDC)wParam_;

			//if(hWnd == GetDlgItem(hDlg_, IDC_EMAIL) || hWnd == GetDlgItem(hDlg_, IDC_URL))
			//{
			//	cfdprintf(_T("WM_CTLCOLORSTATIC"));

			//	SetTextColor(hdc, RGB(0,0,255));
			//	SetBkMode(hdc, TRANSPARENT);
				//SelectObject(hdc, (HBRUSH)GetStockObject(BLACK_BRUSH)); // DC_BRUSH

//				return(TRUE);
			//};
		};
		break;

	case WM_SETCURSOR:
		{
			POINT pt;
			RECT rc1, rc2;

			cfdprintf(_T("WM_SETCURSOR: %d =?= %d"), HIWORD(lParam_), WM_LBUTTONUP);

			GetCursorPos(&pt);

			GetWindowRect(GetDlgItem(hDlg_, IDC_EMAIL), &rc1);
			GetWindowRect(GetDlgItem(hDlg_, IDC_URL), &rc2);

			if(PtInRect(&rc1, pt) || PtInRect(&rc2, pt))
			{
				hOldCur = SetCursor(hHandCur);

				// Clicked...
				if(HIWORD(lParam_) == WM_LBUTTONUP)
				{
					if(PtInRect(&rc1, pt))
					{
						cfdprintf("ABK_DEV_LINK_EMAIL");
						ShellExecute(HWND_DESKTOP, _T("open"), BBLNC_DEV_LINK_EMAIL, NULL, NULL, SW_SHOWNORMAL);
					}
					else
					{
						ShellExecute(HWND_DESKTOP, _T("open"), BBLNC_DEV_LINK_SITE, NULL, NULL, SW_SHOWNORMAL);
					};

					MessageBeep(MB_OK);
				};

				return(TRUE);
			}
			else if(hOldCur)
			{
				SetCursor(hOldCur);
				hOldCur = NULL;
				return(TRUE);
			};
		};
		break;

	case WM_COMMAND:
		if (LOWORD(wParam_) == IDOK || LOWORD(wParam_) == IDCANCEL) 
		{
			EndDialog(hDlg_, LOWORD(wParam_));
			return TRUE;
		};
		break;

	case WM_DESTROY:
		DestroyCursor(hHandCur);
		break;
	};

	return FALSE;
};

LRESULT CMainWindow::_WndProcAboutDlg(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	assert(MainWindow != NULL);
	return(MainWindow->WndProcAboutDlg(hWnd_, uMsg_, wParam_, lParam_));
};

// Message handler proc for license box.
LRESULT CMainWindow::WndProcLicenseDlg(HWND hDlg_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	switch (uMsg_)
	{
	case WM_INITDIALOG:
		{
			HRSRC hrc;
			HGLOBAL hgl;
			LPTSTR szStr, szPtr;
			DWORD dwSize;

			hrc = FindResource((HMODULE)g_hInst, MAKEINTRESOURCE(IDR_GPL_LICENSE), RT_HTML);
			if(hrc == NULL)
				return(FALSE);

			dwSize = SizeofResource((HMODULE)g_hInst, hrc);

			hgl = LoadResource((HMODULE)g_hInst, hrc);
			if(hgl == NULL)
				return(FALSE);

			szPtr = (LPTSTR)LockResource(hgl);
			if(szPtr == NULL)
				return(FALSE);

			szStr = new TCHAR[dwSize + 1];
			memcpy(szStr, szPtr, dwSize);
			szStr[dwSize] = 0;

			SetDlgItemText(hDlg_, IDC_LICENSE_TEXT, szStr);

			delete[] szStr;
			FreeResource(hgl);
		};
		return(TRUE);

	case WM_COMMAND:
		if (LOWORD(wParam_) == IDOK || LOWORD(wParam_) == IDCANCEL) 
		{
			EndDialog(hDlg_, LOWORD(wParam_));
			return(TRUE);
		};
		break;
	};

	return(FALSE);
};

LRESULT CMainWindow::_WndProcLicenseDlg(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	assert(MainWindow != NULL);
	return(MainWindow->WndProcLicenseDlg(hWnd_, uMsg_, wParam_, lParam_));
};

UINT CMainWindow::UpdateDataThread(void* pPrm_)
{
	USES_CONVERSION;

	HRESULT        hr;
	IBotikBalance* pIBB;
	LPTSTR         szVal;
	BSTR           bsVal;

	cfdprintf(_T("CMainWindow::UpdateDataThread"));

	m_fLoop  = TRUE;

	::CoInitialize(NULL);

	hr = CoCreateInstance(__uuidof(BotikBalance), NULL, CLSCTX_INPROC_SERVER,
						  __uuidof(IBotikBalance), (void**)&pIBB);

	if(FAILED(hr))
	{
		::MessageBox(NULL, _T("Error: Internal error, create component failed!"), BBLNC_MSGBOX_CAPTION, MB_OK|MB_ICONHAND);
		cfdprintf(_T("CMainWindow::UpdateDataThread: CoCreateInstance failed"));
		::SendMessage(this->GetHWnd(), WM_DESTROY, 0, 0);
		return(0);
	};

	while(m_fLoop)
	{
		LONG lRet;
		RECT rc;

		g_fUpdateThrdReady = TRUE;

		cfdprintf(_T("CMainWindow::UpdateDataThread: Going to wait event..."));
		lRet = WaitForSingleObject(m_hEvent, INFINITE);

		StopAutoUpdate();

		if(lRet != WAIT_OBJECT_0)
			break;

		if(!m_fLoop)
			break;

		// Setup user's data to connect
		cfdprintf(_T("CMainWindow::UpdateDataThread: Setup user's data to connect..."));
		g_pAppConfigUser->GetValue(BBLNC_ACCOUNTNAME, &szVal, NULL);
		cfdprintf(_T("CMainWindow::UpdateDataThread: ---> Account name: %s"), szVal);
		bsVal = A2BSTR(szVal);
		pIBB->put_AccountName(bsVal);
		delete[] szVal;
		::SysFreeString(bsVal);

		g_pAppConfigUser->GetValue(BBLNC_USER, &szVal, NULL);
		cfdprintf(_T("CMainWindow::UpdateDataThread: ---> User name: %s"), szVal);
		bsVal = A2BSTR(szVal);
		pIBB->put_User(bsVal);
		delete[] szVal;
		::SysFreeString(bsVal);

		g_pAppConfigUser->GetValue(BBLNC_PASSWD, &szVal, NULL);
		cfdprintf(_T("CMainWindow::UpdateDataThread: ---> Passwd name: %s"), szVal);
		bsVal = A2BSTR(szVal);
		pIBB->put_Password(bsVal);
		delete[] szVal;
		::SysFreeString(bsVal);

		g_pAppConfigUser->GetValue(BBLNC_PROXYSERVER, &szVal, (LPTSTR)NULL);
		if(szVal)
		{
			bsVal = A2BSTR(szVal);
			pIBB->put_ProxyServer(bsVal);
			pIBB->put_UseProxy(TRUE);
			delete[] szVal;
			::SysFreeString(bsVal);
		};

		cfdprintf(_T("CMainWindow::UpdateDataThread: Updating data..."));

		pIBB->Connect(&lRet);
		if(lRet != BBO_RET_OK)
		{
			cfdprintf(_T("CMainWindow::UpdateDataThread: Connect failed (%d)"), lRet);
			SetCurrentState(BBLNC_CURRENT_STATE_UPDATE_FAILED);

			if(g_fShowErrors)
			{
				BSTR bsErrTxt;

				pIBB->GetErrorDescription(lRet, &bsErrTxt);

				cfAlertBox(EVENTLOG_ERROR_TYPE, BBLNC_MSGBOX_CAPTION, 
						_T("    Nadmin\n\n : %d\n: %s"),
						lRet, CW2T(bsErrTxt));

				::SysFreeString(bsErrTxt);
			};
		}
		else
		{
			pIBB->Update(&lRet);
			if(lRet != BBO_RET_OK)
			{
				cfdprintf(_T("CMainWindow::UpdateDataThread: Update failed (%d)"), lRet);
				SetCurrentState(BBLNC_CURRENT_STATE_UPDATE_FAILED);

				if(g_fShowErrors)
				{
					BSTR bsErrTxt;

					pIBB->GetErrorDescription(lRet, &bsErrTxt);

					cfAlertBox(EVENTLOG_ERROR_TYPE, BBLNC_MSGBOX_CAPTION, 
							_T("     \n\n : %d\n: %s"),
							lRet, CW2T(bsErrTxt));

					::SysFreeString(bsErrTxt);
				};
			}
			else
			{
				pIBB->Disconnect(&lRet);
				if(lRet != BBO_RET_OK)
				{
					cfdprintf(_T("CMainWindow::UpdateDataThread: Disconnect failed (%d)"), lRet);
					SetCurrentState(BBLNC_CURRENT_STATE_UPDATE_FAILED);

					if(g_fShowErrors)
					{
						BSTR bsErrTxt;

						pIBB->GetErrorDescription(lRet, &bsErrTxt);

						cfAlertBox(EVENTLOG_ERROR_TYPE, BBLNC_MSGBOX_CAPTION, 
								_T("    Nadmin\n\n : %d\n: %s"),
								lRet, CW2T(bsErrTxt));

						::SysFreeString(bsErrTxt);
					};
				}
				else
				{
					BSTR bsName;
					BSTR bsVal;
					COLORREF cref;
					LPTSTR szText;
					LPTSTR szAccount;
					LPTSTR szDate;
					LPTSTR szAccStatus;
					LPTSTR szAccStatusFull;
                    LPTSTR szAccState;
					LPTSTR szAccStateBool;
					LPTSTR szCharge;
					LPTSTR szPPlan;
					LPTSTR szCPFrom;
					LPTSTR szForecast;
                    LPTSTR szOffLevel;
					double nAccStatusNew;
					DWORD  nAccStatus;
					DWORD  nWhenShow;

					cfdprintf(_T("CMainWindow::UpdateDataThread: Data updated successfuly..."));

					g_pAppConfigUser->GetValue(BBLNC_ACCOUNTNAME, &szAccount, NULL);

					bsName = ::SysAllocString(L"AccountInfoDate");
					pIBB->GetData(bsName, &bsVal);
					szDate = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

                    bsName = ::SysAllocString(L"AccountState");
					pIBB->GetData(bsName, &bsVal);
					szAccState = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					bsName = ::SysAllocString(L"AccountStateBool");
					pIBB->GetData(bsName, &bsVal);
					szAccStateBool = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

                    bsName = ::SysAllocString(L"OffLevelFull");
					pIBB->GetData(bsName, &bsVal);
					szOffLevel= CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					bsName = ::SysAllocString(L"AccountStatusFull");
					pIBB->GetData(bsName, &bsVal);
					szAccStatusFull = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					bsName = ::SysAllocString(L"ChargeFull");
					pIBB->GetData(bsName, &bsVal);
					szCharge = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					bsName = ::SysAllocString(L"PricePlanFull");
					pIBB->GetData(bsName, &bsVal);
					szPPlan = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					bsName = ::SysAllocString(L"ChargesPeriodFrom");
					pIBB->GetData(bsName, &bsVal);
					szCPFrom = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					bsName = ::SysAllocString(L"Forecast");
					pIBB->GetData(bsName, &bsVal);
					szForecast = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					bsName = ::SysAllocString(L"AccountStatus");
					pIBB->GetData(bsName, &bsVal);
					szAccStatus = CW2T(bsVal);
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

					nAccStatusNew = atof(szAccStatus);

//					szText = cfsprintf(BBLNC_MSG_FORMAT, szAccount, szDate, szAccStatusFull, szUCharge, szTCharge, szPPlan);
//					NotifyWindow->SetText(szText);

					DWORD dwRow;

					NotifyWindow->ClearTable();

					dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), _T(":"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, RGB(0, 0, 0), szAccount);

					dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), _T("   :"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, RGB(0, 0, 0), szDate);

					cref = RGB(0, 150, 0);
					if(_tcsicmp(szAccStateBool, _T("false")) == 0)
						cref = RGB(200, 0, 0);

                    dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), _T(" :"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, cref, szAccState);

					cref = RGB(0, 0, 0);
					if(nAccStatusNew <= 0)
						cref = RGB(200, 0, 0);

					dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), _T("  :"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, cref, szAccStatusFull);

                    dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), _T("  1 .:"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, RGB(0, 0, 0), szPPlan);

                    double dlStatus;
                    double dlPlan;
                    double dlOff;

                    bsName = ::SysAllocString(L"AccountStatus");
					pIBB->GetData(bsName, &bsVal);
					dlStatus = atof(CW2T(bsVal));
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

                    bsName = ::SysAllocString(L"PricePlan");
					pIBB->GetData(bsName, &bsVal);
					dlPlan = atof(CW2T(bsVal));
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

                    bsName = ::SysAllocString(L"OffLevel");
					pIBB->GetData(bsName, &bsVal);
					dlOff = atof(CW2T(bsVal));
					::SysFreeString(bsVal);
					::SysFreeString(bsName);

                    dlStatus = dlStatus - dlOff;

                    szText = cfsprintf(_T("%s (%.0f .)"), szForecast, (double)abs((double)(dlStatus / dlPlan)));
					dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), _T(":"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, RGB(0, 0, 0), szText);
                    delete[] szText;

                    szText = cfsprintf(_T("  %s:"), szCPFrom);
                    dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), szText);
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, RGB(0, 0, 0), szCharge);
                    delete[] szText;

                    dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, RGB(0, 0, 0), _T(" :"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, RGB(0, 0, 0), szOffLevel);

/* XXX Old 
					szText = cfsprintf(_T("  %s  :"), szCPFrom);
					dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddSeparator(dwRow, NF_DT_LEFT | NF_SEPARATOR_NO_TXT_SIDE_SPACE, szText);
					delete[] szText;

					dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, _T("  -  :"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, szUCharge);

					dwRow = NotifyWindow->AddRow();
					NotifyWindow->AddColumn(dwRow, NF_DT_LEFT, _T("  -    :"));
					NotifyWindow->AddColumn(dwRow, NF_DT_RIGHT, szTCharge);
*/
					NotifyWindow->SetDisplayData(NW_DISPLAY_TABLE_DATA);

					SetCurrentState(BBLNC_CURRENT_STATE_DATA_READY);

					// Check account state...
					g_pAppConfigUser->GetValue(BBLNC_CRITICALACCOUNTSTATE, &nAccStatus, NULL);
					g_pAppConfigUser->GetValue(BBLNC_SHOWINFOWINDOW, &nWhenShow, NULL);

					cfdprintf(_T("CMainWindow::UpdateDataThread: Check for critical situation: New: %f; Level: %d; Pre: %f"), nAccStatusNew, nAccStatus, g_dAccStatusLast);

					m_pNIcon->ChangeText(0, _T("Botik::Balance\n: %s"), szAccStatusFull);
					m_pNIcon->ChangeText(4, _T("Botik::Balance\n: %s"), szAccStatusFull);
					m_pNIcon->ChangeText(5, _T("Botik::Balance\n: %s"), szAccStatusFull);

					if(nWhenShow == IDC_RADIO_STATUS_GET_VALUE && nAccStatusNew < nAccStatus)
					{
						SetCurrentState(BBLNC_CURRENT_STATE_OVER_VALUE);

						cfdprintf(_T("CMainWindow::UpdateDataThread: m_fShowAlarm = %d"), m_fShowAlarm);

						if(m_fShowAlarm)
						{
							NotifyWindow->PopUp();
							m_fShowAlarm = FALSE;
						};
					} 
					else if(nWhenShow == IDC_RADIO_STATUS_CHANGED && nAccStatusNew != g_dAccStatusLast)
					{
						SetCurrentState(BBLNC_CURRENT_STATE_DATA_READY);
						NotifyWindow->PopUp();
					};

					g_dAccStatusLast = nAccStatusNew;

					cfdprintf(_T("CMainWindow::UpdateDataThread: Check for critical situation: New: %f; Level: %d; Pre: %f"), nAccStatusNew, nAccStatus, g_dAccStatusLast);

					delete[] szAccount;
				};
			};
		};
		
		::GetClientRect(NotifyWindow->GetHWnd(), &rc);
		::InvalidateRect(NotifyWindow->GetHWnd(), &rc, TRUE);

		StartAutoUpdate();
	};

	pIBB->Release();

	::CoUninitialize();

	cfdprintf(_T("!CMainWindow::UpdateDataThread"));

	return(1);
};

unsigned int __stdcall CMainWindow::_UpdateDataThread(void* pPrm_)
{
	assert(MainWindow != NULL);
	return(MainWindow->UpdateDataThread(pPrm_));
};