/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <bblnc.h>
#include <window.h>
#include <notifywindow.h>

#define NW_TABLE_ROW_SPACE		0
#define NW_TABLE_COL_SPACE		5
#define NW_CAPTION_SIZE			15
#define NW_CAPTION_TABLE_SPACE	2
#define NW_BORDER_WIDTH			5
#define NW_SEPARATOR_TXT_SPACE	10 // in pixels
#define NW_SEPARATOR_ROW_SPACE	8  // in pixels

CNotifyWindow* NotifyWindow = NULL;

CNotifyWindow::CNotifyWindow(void) : CWindow()
{
	cfdprintf(_T("CNotifyWindow::CNotifyWindow"));
	
	m_lWndWidth    = 0;
	m_lWndHeight   = 0;
	m_fShow        = FALSE;
	m_szText       = NULL;
	m_nTextFlags   = 0;
	m_nDisplInfo   = NW_DISPLAY_TEXT_DATA;

	m_crWndBgColor         = RGB(255, 255, 255);
	m_crWndFgColor         = RGB(0, 0, 0);
	m_crWndFrmNormLColor   = RGB(50, 122, 233); // RGB(132, 183, 233);
	m_crWndFrmNormDColor   = RGB(25, 40, 160);  // RGB(47, 74, 120);
	m_crWndFrmWarnLColor   = RGB(233, 122, 50);
	m_crWndFrmWarnDColor   = RGB(160, 40, 25);
	m_crWndFrmOutlineColor = m_crWndBgColor;

	cfdprintf(_T("!CNotifyWindow::CNotifyWindow"));
};

CNotifyWindow::~CNotifyWindow(void)
{
	cfdprintf(_T("CNotifyWindow::~CNotifyWindow"));
	DeleteCriticalSection(&m_cs);
	cfdprintf(_T("!CNotifyWindow::~CNotifyWindow"));
};

BOOL CNotifyWindow::CreateObject()
{
	assert(NotifyWindow == NULL);
	NotifyWindow = new CNotifyWindow();
	return(TRUE);
};

BOOL CNotifyWindow::Create(HWND hWndPrn_)
{
	BOOL fOk = TRUE;

	cfdprintf(_T("CNotifyWindow::Create"));

	assert(m_hWnd == NULL);

	InitializeCriticalSection(&m_cs);

	ClearTable();

	m_hWnd = ::CreateDialog(g_hInst, MAKEINTRESOURCE(IDD_NOTIFYWND_DLG), NULL, (DLGPROC)_WndProc);

	if(!m_hWnd)
	{
		cfdprintf(_T("CNotifyWindow::Create: CreateDialog failed"));
		fOk = FALSE;
	}
    else if(pSetLayeredWindowAttributes)
    {
        // Windows need to be layered to be made transparent. This is done
        // by modifying the extended style bits to contain WS_EX_LAYARED.
        cfdprintf(_T("CNotifyWindow::Create: We are running on the system with transparent window ability, so init it!"));
        ::SetWindowLong(m_hWnd, GWL_EXSTYLE, ::GetWindowLong(m_hWnd, GWL_EXSTYLE)|WS_EX_LAYERED);
    };

	cfdprintf(_T("!CNotifyWindow::Create"));

	return(fOk);
};

BOOL CNotifyWindow::SetWindowPosition()
{
	RECT rcScr;

	::SystemParametersInfo(SPI_GETWORKAREA, 0, &rcScr, 0);

	this->MoveTo(rcScr.right-m_lWndWidth-1, rcScr.bottom-m_lWndHeight, m_lWndWidth, m_lWndHeight);

	return(TRUE);
};

VOID  CNotifyWindow::SetDisplayData(UINT nType_)
{
	RECT rc;

	m_nDisplInfo = nType_;

	::GetClientRect(m_hWnd, &rc);
	::InvalidateRect(m_hWnd, &rc, TRUE);
};

VOID CNotifyWindow::PopUp()
{
	RECT  rc;
	DWORD dwTimeOut;

	if(m_fShow) return;

	m_fShow = TRUE;
	g_pAppConfigUser->GetValue(BBLNC_INFOWINDOWTIMEOUT, &dwTimeOut, 0);
	Show(SW_SHOW);

	::UpdateWindow(m_hWnd);
	::GetClientRect(m_hWnd, &rc);
	::InvalidateRect(m_hWnd, &rc, TRUE);

	if(dwTimeOut > 0)
		::SetTimer(m_hWnd, NOTIFYWND_TIMER_ID, dwTimeOut*1000, (TIMERPROC)NULL);
};

VOID CNotifyWindow::Hide()
{
	m_fShow = FALSE;
	::KillTimer(m_hWnd, NOTIFYWND_TIMER_ID);
	Show(SW_HIDE);
};

VOID CNotifyWindow::SetText(UINT nFlags_, LPTSTR szText_)
{
	if(m_szText)
		delete[] m_szText;

	m_szText = new TCHAR[_tcslen(szText_)+1];
	_tcscpy(m_szText, szText_);

	m_nTextFlags = nFlags_;
};

VOID CNotifyWindow::DrawFrame(HDC hdc_, CONST LPRECT prc_, COLORREF cr1_, COLORREF cr2_)
{
	HPEN		hPen, hPenOld;

	hPen = ::CreatePen(PS_SOLID, 2, cr1_);
	hPenOld = (HPEN)::SelectObject(hdc_, hPen);

	::MoveToEx(hdc_, prc_->left+1, prc_->bottom, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->left+1, prc_->top);
	::LineTo(hdc_, prc_->right, prc_->top+1);

	::MoveToEx(hdc_, prc_->left, prc_->bottom-3, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->right-3, prc_->bottom-3);
	::LineTo(hdc_, prc_->right-3, prc_->top);
	
	hPen = ::CreatePen(PS_SOLID, 2, cr2_);
	::DeleteObject(::SelectObject(hdc_, hPen));

	::MoveToEx(hdc_, prc_->left+3, prc_->bottom-4, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->left+3, prc_->top+2);
	::LineTo(hdc_, prc_->right-4, prc_->top+3);

	::MoveToEx(hdc_, prc_->left+1, prc_->bottom-1, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->right-1, prc_->bottom-1);
	::LineTo(hdc_, prc_->right-1, prc_->top+2);

	::SetPixel(hdc_, prc_->left+1, prc_->bottom-1, cr2_);
	::SetPixel(hdc_, prc_->left+2, prc_->bottom-3, cr2_);
	::SetPixel(hdc_, prc_->right-1, prc_->top+1, cr2_);
	::SetPixel(hdc_, prc_->right-3, prc_->top+2, cr2_);

	hPen = ::CreatePen(PS_SOLID, 1, m_crWndFrmOutlineColor);
	::DeleteObject(::SelectObject(hdc_, hPen));

	::MoveToEx(hdc_, prc_->left+4, prc_->bottom-5, (LPPOINT)NULL);
	::LineTo(hdc_, prc_->left+4, prc_->top+4);
	::LineTo(hdc_, prc_->right-5, prc_->top+4);
	::LineTo(hdc_, prc_->right-5, prc_->bottom-5);
	::LineTo(hdc_, prc_->left+4, prc_->bottom-5);

	::DeleteObject(::SelectObject(hdc_, hPenOld));
};

VOID CNotifyWindow::DisplayTableData(HDC hdc_)
{
	RECT  crc;
	DWORD dwCx = 0;
	DWORD dwRowHeight = 0;
	DWORD dwRowTxtCnt = 0;
	DWORD dwRowSepCnt = 0;
	INTVECTOR ivColSize;
	LONG  lMostLongSep = 0;

	// GetTextExtentPoint32
	// DrawText

	cfdprintf(_T("CNotifyWindow::DisplayInformation"));
	cfdprintf(_T("CNotifyWindow::DisplayInformation: Rows: %d"), m_nftContent.size());

	// Get sizes in pixels of most lage strings in columns.
	for(NFTABLEITR it = m_nftContent.begin(); it != m_nftContent.end(); it++)
	{
		DWORD dwCol = 0;
		LPNFTABLECELL ptr = (LPNFTABLECELL)*it;

		while(ptr)
		{
			SIZE size;

			BOOL fRet = GetTextExtentPoint32(hdc_, ptr->text, (INT)_tcslen(ptr->text), &size);

			if(ptr->type == TypeSeparator)
			{
				cfdprintf("Sep: %d; cx: %d; cy: %d", fRet, size.cx, size.cy);

				if(lMostLongSep < size.cx)
					lMostLongSep = size.cx;

				dwRowHeight = size.cy;
				dwRowSepCnt++;
			}
			else
			{
				dwRowHeight = size.cy;

				// only first cell count.
				if(ptr->prev == NULL)
					dwRowTxtCnt++;

				if(ivColSize.size() <= dwCol)
				{
					ivColSize.push_back(size.cx);
					dwCx += size.cx;
				}
				else
				{
					if(ivColSize.at(dwCol) < size.cx)
					{
						dwCx -= ivColSize[dwCol];
						dwCx += size.cx;
						ivColSize[dwCol] = size.cx;
					};
				};
			};

			ptr = ptr->next;
			dwCol++;
		};
	};

	// Calc new window size.
	DWORD dwTmp1, dwTmp2;

	dwTmp1 = lMostLongSep + (NW_SEPARATOR_TXT_SPACE * 2) + (2 * NW_BORDER_WIDTH)/* + 1*/;
	dwTmp2 = dwCx + (((INT)ivColSize.size() - 1) * NW_TABLE_COL_SPACE) + 2 * NW_BORDER_WIDTH/* + 1*/;

	m_lWndWidth  = dwTmp1 > dwTmp2 ? dwTmp1 : dwTmp2;
	m_lWndHeight = ((dwRowHeight + NW_TABLE_ROW_SPACE) * dwRowTxtCnt) + ((dwRowHeight + NW_TABLE_ROW_SPACE + NW_SEPARATOR_ROW_SPACE) * dwRowSepCnt) + NW_TABLE_ROW_SPACE;
	m_lWndHeight += NW_CAPTION_SIZE + NW_CAPTION_TABLE_SPACE + 2 * NW_BORDER_WIDTH;

	SetWindowPosition();

	cfdprintf(_T("CNotifyWindow::DisplayInformation: cx: %d; cy: %d; rh: %d; tn: %d; sn: %d"), 
			  m_lWndWidth, m_lWndHeight, dwRowHeight, dwRowTxtCnt, dwRowSepCnt);

	::GetClientRect(m_hWnd, &crc);
	crc.top  = NW_CAPTION_SIZE + NW_CAPTION_TABLE_SPACE + NW_BORDER_WIDTH;
	crc.left = NW_BORDER_WIDTH;

	// write down table.
	RECT rcTxt = crc;

	for(NFTABLEITR it = m_nftContent.begin(); it != m_nftContent.end(); it++)
	{
		DWORD dwCol  = 0;
		LPNFTABLECELL ptr = (LPNFTABLECELL)*it;
		DWORD nFlags;
		COLORREF cref;

		rcTxt.left = NW_BORDER_WIDTH;

		if(ptr->type == TypeSeparator)
		{
			rcTxt.top += NW_SEPARATOR_ROW_SPACE;
			rcTxt.right = m_lWndWidth;

			nFlags = ptr->flags & 0x000000ff; // for DrawText

			// draw text
			if(!(ptr->flags & NF_SEPARATOR_NO_TXT_SIDE_SPACE))
			{
				rcTxt.left += NW_SEPARATOR_TXT_SPACE + 2;
				rcTxt.right -= (NW_SEPARATOR_TXT_SPACE + NW_BORDER_WIDTH);
			};

			cref = ::SetTextColor(hdc_, ptr->cref);
			::DrawText(hdc_, ptr->text, (INT)_tcslen(ptr->text), &rcTxt, nFlags);
			if(cref != CLR_INVALID)
				::SetTextColor(hdc_, cref);

			rcTxt.top += dwRowHeight + NW_TABLE_ROW_SPACE; // XXX we already add separator row space
		}
		else
		{
			while(ptr)
			{
				rcTxt.right = ivColSize[dwCol] + rcTxt.left;

				nFlags = ptr->flags & 0x000000ff; // for DrawText

				cref = ::SetTextColor(hdc_, ptr->cref);
				::DrawText(hdc_, ptr->text, (INT)_tcslen(ptr->text), &rcTxt, nFlags);
				if(cref != CLR_INVALID)
					::SetTextColor(hdc_, cref);

				ptr = ptr->next;

				rcTxt.left += ivColSize[dwCol] + NW_TABLE_COL_SPACE;

				dwCol++;
			};

			rcTxt.top += dwRowHeight + NW_TABLE_ROW_SPACE;
		};
	};

	cfdprintf(_T("!CNotifyWindow::DisplayInformation"));
};

DWORD CNotifyWindow::AddRow()
{
	cfdprintf(_T("CNotifyWindow::AddRow"));

	DWORD dwRow;

	EnterCriticalSection(&m_cs);

	cfdprintf(_T("CNotifyWindow::AddRow: Currently stored rows: %d"), m_nftContent.size());

	dwRow = (DWORD)m_nftContent.size();
	m_nftContent.push_back(NULL);
	
	LeaveCriticalSection(&m_cs);
	
	cfdprintf(_T("!CNotifyWindow::AddRow: New row number: %d"), dwRow);

	return(dwRow);
};

DWORD CNotifyWindow::AddColumn(DWORD dwRow_, UINT nFalgs_, COLORREF cref_, LPTSTR szFormat_, ...)
{
	DWORD dwCol = 0;
	DWORD dwSize;
	DWORD dwRet = 1;

	cfdprintf(_T("CNotifyWindow::AddColumn"));

	assert(m_nftContent.size() > dwRow_);

	LPTSTR szTxt = szFormat_;  // XXX
	
	EnterCriticalSection(&m_cs);

	LPNFTABLECELL ptr = new NFTABLECELL;

	dwSize = (DWORD)_tcslen(szTxt);
	ptr->text = new TCHAR[dwSize + 1];
	_tcscpy(ptr->text, szTxt);
	ptr->flags = nFalgs_;
	ptr->type  = TypeText;
	ptr->next  = NULL;
	ptr->cref  = cref_;

	LPNFTABLECELL cell = m_nftContent.at(dwRow_);

//	__asm int 3;

	if(cell == NULL)
	{
		ptr->prev = NULL;
		m_nftContent[dwRow_] = ptr;
	}
	else
	{
		dwCol = 1;

		if(cell->type == TypeSeparator)
		{
			cfdprintf(_T("CNotifyWindow::AddColumn: ERR: This row is seporator only."));
			delete[] ptr->text;
			delete ptr;
			dwRet = 0;
			goto exit;
		};

		while(cell->next) 
		{
			cfdprintf(_T("CNotifyWindow::AddColumn: Looking for the tail... Current: %d (%s)"), dwCol, cell->text);

			cell = cell->next;
			dwCol++;
		};

		cell->next = ptr;
		ptr->prev = cell;
	};

	cfdprintf(_T("CNotifyWindow::AddColumn: Added %d column in the row %d"), dwCol, dwRow_);
	
exit:
	LeaveCriticalSection(&m_cs);
	
	cfdprintf(_T("!CNotifyWindow::AddColumn"));
	
	return(dwRet);
};

DWORD CNotifyWindow::AddSeparator(DWORD dwRow_, UINT nFalgs_, COLORREF cref_, LPTSTR szFormat_, ...)
{
	DWORD dwRet = 1;
	DWORD dwSize;

	cfdprintf(_T("CNotifyWindow::AddSeparator"));

	LPTSTR szTxt = szFormat_;  // XXX

	EnterCriticalSection(&m_cs);

	LPNFTABLECELL cell = m_nftContent.at(dwRow_);

	if(cell != NULL)
	{
		cfdprintf(_T("CNotifyWindow::AddSeparator: ERR: This row is not empty."));
		dwRet = 0;
		goto exit;
	}
	else
	{
		LPNFTABLECELL ptr = new NFTABLECELL;

		dwSize = (DWORD)_tcslen(szTxt);
		ptr->text = new TCHAR[dwSize + 1];
		_tcscpy(ptr->text, szTxt);
		ptr->flags = nFalgs_;
		ptr->type  = TypeSeparator;
		ptr->cref  = cref_;
		ptr->next  = NULL;
		ptr->prev  = NULL;
		m_nftContent[dwRow_] = ptr;
	};

exit:
	LeaveCriticalSection(&m_cs);

	cfdprintf(_T("!CNotifyWindow::AddSeparator"));

	return(dwRet);
};

VOID CNotifyWindow::ClearTable()
{
	DWORD dwRow;
	DWORD dwCol;

	cfdprintf(_T("CNotifyWindow::ClearTable"));

	EnterCriticalSection(&m_cs);

	dwRow = 0;

	for(NFTABLEITR it = m_nftContent.begin(); it != m_nftContent.end(); it++)
	{
		LPNFTABLECELL ptr = (LPNFTABLECELL)*it;

		dwCol = 0;

		while(ptr)
		{
			LPNFTABLECELL next = ptr->next;

			cfdprintf(_T("CNotifyWindow::ClearTable: Free Row: %d; Col: %d (%s)"), dwRow, dwCol, ptr->text);

			delete[] ptr->text;
			delete ptr;
			ptr = next;

			dwCol++;
		};

		dwRow++;
	};

	m_nftContent.clear();

	LeaveCriticalSection(&m_cs);

	cfdprintf(_T("!CNotifyWindow::ClearTable"));
};

///////////////////////////////////////////////////////////////////////////////
// WM event handlers
///////////////////////////////////////////////////////////////////////////////

LRESULT CNotifyWindow::OnInitDialog(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnInitDialog"));

	m_hWnd       = hWnd_;
	m_lWndWidth  = 220;
	m_lWndHeight = 110;

	this->SetWindowPosition();

	cfdprintf(_T("!CNotifyWindow::OnInitDialog"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnPaint(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	PAINTSTRUCT ps;
	HDC         hdc;
	RECT        rc, rcCapt, rcTxt;
	HBRUSH      hBr;
	COLORREF	crCapt;
	LOGFONT		lf;
	HFONT       hFont;
	LONG		lBgLogoID;
	BITMAP		bm;
	HBITMAP		hBmp;
	HDC			hMemDc;
	UINT		nFlags;
	HBITMAP     hBmpOld;
	HFONT		hFontOld;

	cfdprintf(_T("CNotifyWindow::OnPaint"));

	hdc = ::BeginPaint(hWnd_, &ps);

	::GetClientRect(hWnd_, &rc);

	// Fill window background
	hBr = ::CreateSolidBrush(m_crWndBgColor);
	::FillRect(hdc, &rc, hBr);
	::DeleteObject(hBr);

	// Draw background image
	lBgLogoID = g_nCurState == BBLNC_CURRENT_STATE_OVER_VALUE ? IDB_BGLOGO_WARN : IDB_BGLOGO_NORM;
	hBmp = (HBITMAP)::LoadImage(g_hInst, MAKEINTRESOURCE(lBgLogoID), IMAGE_BITMAP, 0, 0, LR_DEFAULTCOLOR|LR_CREATEDIBSECTION);
	if(hBmp == NULL)
		return(FALSE);

	::GetObject(hBmp, sizeof(BITMAP), &bm);

	hMemDc = ::CreateCompatibleDC(hdc);
	hBmpOld = (HBITMAP)::SelectObject(hMemDc, hBmp);
	::SetMapMode(hMemDc, ::GetMapMode(hdc));
	::BitBlt(hdc, rc.right-bm.bmWidth-5, rc.bottom-bm.bmHeight-5, bm.bmWidth, bm.bmHeight, hMemDc, 0, 0, SRCCOPY);

	::DeleteObject(::SelectObject(hMemDc, hBmpOld));
	::DeleteObject(hBmp);
	::DeleteDC(hMemDc);

	// Draw caption
	if(g_nCurState != BBLNC_CURRENT_STATE_OVER_VALUE)
		crCapt = m_crWndFrmNormLColor;
	else
		crCapt = m_crWndFrmWarnLColor;

	rcCapt = rc;
	rcCapt.bottom = NW_CAPTION_SIZE + NW_BORDER_WIDTH; // XXX good to know font height and not using define for capt size
	hBr = ::CreateSolidBrush(crCapt);
	::FillRect(hdc, (LPRECT)&rcCapt, hBr);
	::DeleteObject(hBr);

	// Draw window frame
	if(g_nCurState != BBLNC_CURRENT_STATE_OVER_VALUE)
		this->DrawFrame(hdc, &rc, m_crWndFrmNormLColor, m_crWndFrmNormDColor);
	else
		this->DrawFrame(hdc, &rc, m_crWndFrmWarnLColor, m_crWndFrmWarnDColor);

	// Get default GUI font information.
	::GetObject(::GetStockObject(DEFAULT_GUI_FONT), sizeof(LOGFONT), &lf);
	lf.lfWeight = FW_BOLD;
	lf.lfHeight = 12;
//	lf.lfWidth  = 5;
	lf.lfQuality = PROOF_QUALITY;
	hFont = ::CreateFontIndirect(&lf);
	hFontOld = (HFONT)::SelectObject(hdc, hFont);

	::SetBkMode(hdc, TRANSPARENT);

	// Write out caption
	LPTSTR szTxt = _T("  ");
	rcTxt = rcCapt;
	rcTxt.left += 7;
	rcTxt.top += 6;
	rcTxt.right -= 5;
	::SetTextColor(hdc, m_crWndBgColor);
	::DrawText(hdc, szTxt, (INT)_tcslen(szTxt), &rcTxt, DT_CENTER);

	::DeleteObject(::SelectObject(hdc, hFontOld));

	// Write out caption
	if(m_szText || m_nftContent.size())
	{
		szTxt = m_szText;
		nFlags = m_nTextFlags;
		lf.lfWeight = FW_NORMAL;
		rcTxt.top = rcCapt.bottom + 4;  // XXX chage to manage window size depends of text size
	}
	else
	{
		szTxt = _T(" ...");
		nFlags = DT_CENTER;
		lf.lfWeight = FW_BOLD;
		rcTxt.top = rcCapt.bottom + 30;
		m_nDisplInfo = NW_DISPLAY_TEXT_DATA;
	};

	rcTxt.bottom = rc.bottom - 6;
	
//	lf.lfHeight = 11;
//	lf.lfWidth  = 6;
	hFont = CreateFontIndirect(&lf);
	hFontOld = (HFONT)::SelectObject(hdc, hFont);

	::SetTextColor(hdc, m_crWndFgColor);

	if(m_nDisplInfo == NW_DISPLAY_TABLE_DATA)
		DisplayTableData(hdc);
	else
		::DrawText(hdc, szTxt, (INT)_tcslen(szTxt), &rcTxt, nFlags);

	::DeleteObject(::SelectObject(hdc, hFontOld));

	::EndPaint(hWnd_, &ps);

	cfdprintf(_T("!CNotifyWindow::OnPaint"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnDestroy(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnDestroy"));

	::KillTimer(hWnd_, NOTIFYWND_TIMER_ID);
	PostQuitMessage(0);

	cfdprintf(_T("!CNotifyWindow::OnDestroy"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnCommand(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnCommand"));

	LRESULT lRes = FALSE;
	INT wmId     = LOWORD(wParam_);
	INT wmEvent  = HIWORD(wParam_);

//	switch(wmId)
//	{
//		// Menu commands...
//	};

	cfdprintf(_T("!CNotifyWindow::OnCommand (lRes = %d)"), lRes);

	return(lRes);
};

LRESULT CNotifyWindow::OnNotify(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	cfdprintf(_T("CNotifyWindow::OnNotify"));
	cfdprintf(_T("!CNotifyWindow::OnNotify"));

	return(TRUE);
};

LRESULT CNotifyWindow::OnTimer(HWND hWnd_, WPARAM wParam_, LPARAM lParam_)
{
	if(wParam_ == NOTIFYWND_TIMER_ID)
		this->Hide();

	return(TRUE);
};

///////////////////////////////////////////////////////////////////////////////
// Window procedures
///////////////////////////////////////////////////////////////////////////////

LRESULT CNotifyWindow::WndProc(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	LRESULT lRes = 0;

//	cfdprintf(_T("CNotifyWindow::WndProc"));

	switch(uMsg_)
	{
		case WM_INITDIALOG:
			lRes = OnInitDialog(hWnd_, wParam_, lParam_);
			break;

		case WM_PAINT:
			lRes = OnPaint(hWnd_, wParam_, lParam_);
			break;

		case WM_DESTROY:
			lRes = OnDestroy(hWnd_, wParam_, lParam_);
			break;

		case WM_COMMAND:
			lRes = OnCommand(hWnd_, wParam_, lParam_);
			break;

		case WM_NOTIFY:
			lRes = OnNotify(hWnd_, wParam_, lParam_);
			break;

		case WM_TIMER:
			lRes = OnTimer(hWnd_, wParam_, lParam_);
			break;

		case WM_LBUTTONUP:
			this->Hide();
			break;

		default:
			break;
	};

//	cfdprintf(_T("!CNotifyWindow::WndProc (lRes = %d)"), lRes);

	return(lRes);
};

LRESULT CNotifyWindow::_WndProc(HWND hWnd_, UINT uMsg_, WPARAM wParam_, LPARAM lParam_)
{
	assert(NotifyWindow != NULL);
	return(NotifyWindow->WndProc(hWnd_, uMsg_, wParam_, lParam_));
};

