/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

var PORT_HTTP  = 80;
var PORT_HTTPS = 443;

var HTTP_GET   = 0;
var HTTP_POST  = 1;
var HTTP_PUT   = 2;

var BBO_RET_OK                = 0;		// OK status
var BBO_RET_FAIL              = -1;		// Unecspected error occured
var BBO_RET_INETOPEN          = -2;		// Open internet session failed
var BBO_RET_INETCONN          = -3;		// Connect operation failed
var BBO_RET_OPENREQ           = -4;		// Open request operation failed
var BBO_RET_SENDREQ           = -5;		// Send request operation failed
var BBO_RET_INETREAD          = -6;		// Read operation failed
var BBO_RET_WRONGMETHOD       = -7;		// Wrong HTTP method
var BBO_RET_WRONGPROXY        = -8;		// Incorrect proxy address (must be address:port)
var BBO_RET_PARSEFAILED       = -9;		// Parsing data failed
var BBO_RET_LOGIN_BADPSWD     = -10;	// Nadmin login failed, bad password
var BBO_RET_LOGIN_BADUSER     = -11;	// Nadmin login failed, bad user name
var BBO_RET_LOGIN_BADACCOUNT  = -12;	// Nadmin login failed, bad account name
var BBO_RET_LOGIN_NOTLOGGEDIN = -13;	// Nadmin login page
var BBO_RET_LOGIN_BADPARAMS   = -14;    // Nadmin login failed, at least one login param is empty

var VERSION = "0.0.9";

BotikBalance.DbgPrint("BotikBalance Parser ver. "+VERSION);

BotikBalance.onConnect    = onConnectHandler;
BotikBalance.onUpdate     = onUpdateHandler;
BotikBalance.onDisconnect = onDisconnectHandler;

function onConnectHandler()
{
    var strUrl  = "";
    var strData = "";
	var strPage = "";
	var intOk   = BBO_RET_FAIL;
	
	BotikBalance.DbgPrint("onConnectHandler fired!!!");
	
	BotikBalance.DbgPrint("onConnectHandler: Account:  "+BotikBalance.AccountName);
	BotikBalance.DbgPrint("onConnectHandler: User:     "+BotikBalance.User);
	BotikBalance.DbgPrint("onConnectHandler: Password: "+BotikBalance.Password);
	
	BotikBalance.DbgPrint("onConnectHandler: Connecting to the Nadmin service...");
	
	strUrl  = "/user/xuser.cgi";
	strData = "do_auth=1&ORG="+encodeURIComponent(BotikBalance.AccountName)+
              "&LOGIN="+encodeURIComponent(BotikBalance.User)+"&PASSWORD="+encodeURIComponent(BotikBalance.Password);
    
    BotikBalance.DbgPrint("onConnectHandler: URL: "+strUrl+"; DATA: "+strData);
    
    // nadmin's method GET handler is unstable, so, i decided to use POST
//	strPage = BotikBalance.RequestPage(HTTP_GET, "nadmin.botik.ru", PORT_HTTPS, strUrl, "", "", true);
	strPage = BotikBalance.RequestPage(HTTP_POST, "nadmin.botik.ru", PORT_HTTPS, strUrl, strData, "", true);
	
	intOk = BotikBalance.LastOperationCode;
	
	if(intOk != BBO_RET_OK)
		BotikBalance.DbgPrint("onConnectHandler: Connect to the Nadmin service... failed ("+intOk+")");
	else
	{
		BotikBalance.DbgPrint("onConnectHandler: Connect to the Nadmin service... succeeded");
		
		// Check if login success...
		intOk = CheckIsLoggedOn(strPage);
	};
	
	BotikBalance.DbgPrint("!onConnectHandler ("+intOk+")");
	
	return(intOk);
};

function onUpdateHandler()
{
	var strPageCurAcc = null;
	var strPageCurPrc = null;
	var oData   = "";
	var intOk   = BBO_RET_FAIL;
	
	BotikBalance.DbgPrint("onUpdateHandler fired!!!");

	BotikBalance.DbgPrint("onUpdateHandler: Updating data from Nadmin service...");	
	
	BotikBalance.DbgPrint("onUpdateHandler: Updating account status...");

	strPageCurAcc = BotikBalance.RequestPage(HTTP_GET, "nadmin.botik.ru", PORT_HTTPS, 
									         "/user/xuser.cgi?do_show_summary_status=1", "", "", true);
    
	try
	{
		intOk = BotikBalance.LastOperationCode;
		
		if(intOk != BBO_RET_OK)
			throw new CException(0, "Update from Nadmin service... failed ("+intOk+")");
		BotikBalance.DbgPrint("onUpdateHandler: Update call to Nadmin service... succeeded");
		
		// Check if logged in
		intOk = CheckIsLoggedOn(strPageCurAcc);
		if(intOk != BBO_RET_OK)
			throw new CException(0, "CheckIsLoggedOn false");
		
		intOk = BBO_RET_PARSEFAILED;
		
		oData = GetAccountStatus(strPageCurAcc);
		if(oData == null)
			throw new CException(0, "GetAccountStatus failed...");
		BotikBalance.SetData("AccountStatusFull", oData.value+" "+oData.unit);
		BotikBalance.SetData("AccountStatus", oData.value);
		
		oData = GetCharge(strPageCurAcc);
		if(oData == null)
			throw new CException(0, "GetCharge failed...");
		BotikBalance.SetData("ChargeFull", oData.value+" "+oData.unit);
		BotikBalance.SetData("Charge", oData.value);
				
		oData = GetAccountInfoDate(strPageCurAcc);
		if(oData == null)
			throw new CException(0, "GetAccountInfoDate failed...");
		BotikBalance.SetData("AccountInfoDate", oData);
		
		oData = GetPricePlan(strPageCurAcc);
		if(oData == null)
			throw new CException(0, "GetPricePlan failed...");
		BotikBalance.SetData("PricePlanFull", oData.value+" "+oData.unit);
		BotikBalance.SetData("PricePlan", oData.value);
		
		oData = GetChargesPeriod(strPageCurAcc);
		if(oData == null)
			throw new CException(0, "GetChargesPeriod failed...");
		BotikBalance.SetData("ChargesPeriodFrom", oData);
		
		oData = GetForecast(strPageCurAcc);
		if(oData == null)
			BotikBalance.SetData("Forecast", "");
		else
			BotikBalance.SetData("Forecast", oData);
		
		oData = GetOffLevel(strPageCurAcc);
		if(oData == null)
			throw new CException(0, "GetOffLevel failed...");
		BotikBalance.SetData("OffLevelFull", oData.value+" "+oData.unit);
		BotikBalance.SetData("OffLevel", oData.value);
		
		oData = GetAccountState(strPageCurAcc);
		BotikBalance.SetData("AccountState", oData.text);
		BotikBalance.SetData("AccountStateBool", oData.state);
		
		intOk = BBO_RET_OK;
	}
	catch(e)
	{
		BotikBalance.DbgPrint("onUpdateHandler: catch: "+e.description+"; OpCode: "+intOk);
	}
	finally
	{
		strPage = null;
	};
	
	BotikBalance.DbgPrint("!onUpdateHandler ("+intOk+")");
	
	return(intOk);
};

function onDisconnectHandler()
{
	var intOk = BBO_RET_FAIL;
	
	BotikBalance.DbgPrint("onDisconnectHandler fired!!!");
	
	BotikBalance.DbgPrint("onDisconnectHandler: Disconnecting from the Nadmin service...");
	strPage = BotikBalance.RequestPage(HTTP_GET, "nadmin.botik.ru", PORT_HTTPS, 
									   "/user/xuser.cgi?do_exit=1", "", "", true);
									   
	intOk = BotikBalance.LastOperationCode;
	
	if(intOk != BBO_RET_OK)
		BotikBalance.DbgPrint("onDisconnectHandler: Disconnect from the Nadmin service... failed ("+intOk+")");
	else
		BotikBalance.DbgPrint("onDisconnectHandler: Disconnect from the Nadmin service... succeeded");
	
	BotikBalance.DbgPrint("!onDisconnectHandler ("+intOk+")");
	
	return(intOk);
};

//////////////////////////////////////////////////////////////////
// Data parsing routines.
//////////////////////////////////////////////////////////////////

function CheckIsLoggedOn(strPage_)
{
	if(strPage_.search(/^:/img) == -1) // caption
		return(BBO_RET_OK);
	else
	{
		var re;

		// Check for wrong password...
	    re = /:    /img;
		if(re.exec(strPage_))
			return(BBO_RET_LOGIN_BADPSWD);
		
		// Check for wrong user name...
//		re = /oe   [^\s]+ ,     /img;
		re = /:  [^\s]+   [^\s]+  /img;
		if(re.exec(strPage_))
			return(BBO_RET_LOGIN_BADUSER);
		
		// Check for wrong account name
		re = /" ":      /img;
		if(re.exec(strPage_))
			return(BBO_RET_LOGIN_BADACCOUNT);
		
		// Check for wrong account name
		re = /:         LOGIN, PASSWORD  ORG/img;
		if(re.exec(strPage_))
			return(BBO_RET_LOGIN_BADPARAMS);	

		return(BBO_RET_LOGIN_NOTLOGGEDIN);
	};
	
	return(BBO_RET_PARSEFAILED);
};

// https://nadmin.botik.ru/xuser.cgi?do_show_price_plan=1
function GetPricePlan(strPage_)
{
	var oRet = null;
	var re   = null;
	var st   = null;

	re = /mb_cost: ([0-9]{1,}\.[0-9]{1,})/img;
	
	if(st = re.exec(strPage_))
	{
		oRet = new CMonetary(st[1], ".");
		BotikBalance.DbgPrint("GetPricePlan: "+oRet.Dump());
	};

	return(oRet);
};

// For page /xuser.cgi?do_show_current_account=1
function GetAccountStatus(strPage_)
{
	var oRet = null;
	var re   = null;
	var st   = null;

	re = /money_left: ([-0-9]{1,}\.[0-9]{1,})/img;
	
	if(st = re.exec(strPage_))
	{
		oRet = new CMonetary(st[1], ".");
		BotikBalance.DbgPrint("GetAccountStatus: "+oRet.Dump());
	};

	return(oRet);
};

function GetCharge(strPage_)
{
	var oRet = null;
	var re   = null;
	var st   = null;
	
	re = /spent: ([0-9]{1,}\.[0-9]{1,})/img;
	
	if(st = re.exec(strPage_))
	{
		oRet = new CMonetary(st[1], ".");
		BotikBalance.DbgPrint("GetCharge: "+oRet.Dump());
	};
	
	return(oRet);
};

function GetAccountInfoDate(strPage_)
{
	var strRet = null;
	var re     = null;
	var st     = null;
	
	re = /date_now: ([0-9]{1,})-([0-9]{1,})-([0-9]{1,})-([0-9]{1,})-([0-9]{1,})-([0-9]{1,})/img;
	
	if(st = re.exec(strPage_))
	{
		strRet = st[3]+"/"+st[2]+"/"+st[1]+" "+st[4]+":"+st[5]+":"+st[6];
		BotikBalance.DbgPrint("GetAccountInfoDate: strRet: "+strRet);
	};
	
	return(strRet);
};

function GetChargesPeriod(strPage_)
{
	var strRet = null;
	var re     = null;
	var st     = null;
	
	re = /period_start: ([0-9]{1,})-([0-9]{1,})-([0-9]{1,})-([0-9]{1,})-([0-9]{1,})-([0-9]{1,})/img;
	
	if(st = re.exec(strPage_))
	{
		strRet = st[3]+"/"+st[2]+"/"+st[1];//+" "+st[4]+":"+st[5]+":"+st[6];
		BotikBalance.DbgPrint("GetChargesPeriod: strRet: "+strRet);
	};
	
	return(strRet);
};

function GetForecast(strPage_)
{
	var strRet = null;
	var re     = null;
	var st     = null;
	
    re = /left_days: ([0-9]{1,})/img;
	
	if(st = re.exec(strPage_))
	{
	    var strDay = "";
		var strLast = new String(st[1]);
	    var strLast = strLast.charAt(strLast.length - 1);
	    
	    if(strLast == 1)
	        strDay = "";
	    else if(strLast == 2 || strLast == 3 || strLast == 4)
	        strDay = "";
	    
		strRet = st[1]+" "+strDay;
		BotikBalance.DbgPrint("GetForecast: strRet: "+strRet);
	};
	
	return(strRet);
};

function GetOffLevel(strPage_)
{
	var oRet = null;
	var re   = null;
	var st   = null;

//    re = /offlevel: ([-0-9]{1,}\.[0-9]{1,})/img;
	re = /offlevel: ([0-9\.-]{1,})/img;

	if(st = re.exec(strPage_))
	{
		oRet = new CMonetary(st[1], ".");
		BotikBalance.DbgPrint("GetOffLevel: "+oRet.Dump());
	};

	return(oRet);
};

function GetAccountState(strPage_)
{
	var oRet = new Object();
	var re   = null;
	var st   = null;

    re = /^disabled: ([^\n\r]{1,})$/img;
    
    oRet.text  = "";
    oRet.state = true;

	if(st = re.exec(strPage_))
	{
//		oRet.text  = " ("+st[1]+")";
        oRet.text  = "";
        oRet.state = false;
		BotikBalance.DbgPrint("GetAccountState: oRet.text: "+oRet.text+"; oRet.state: "+oRet.state);
	};

	return(oRet);
};

//////////////////////////////////////////////////////////////////
// Data objects.
//////////////////////////////////////////////////////////////////

function CException(code_, descr_)
{
	this.code = code_ ? code_ : null;
	this.description = descr_ ? descr_ : null;
};

function CMonetary(value_, unit_)
{
	this.value    = value_ ? new Number(value_) : null;
	this.unit     = unit_ ? unit_ : null;
	this.Dump     = __MonetaryDumpImpl;
	this.toString = __MonetaryToStringImpl;
};

function __MonetaryDumpImpl()
{
	return("Object.value: "+this.value+"; Object.unit: "+this.unit);
};

function __MonetaryToStringImpl()
{
	return(this.value+" "+this.unit);
};
