/*  BotikBalance - User's account balance informer for Botik Network 
 *                 for Windows 9x/ME/NT/2000/XP/2003
 *  Copyright (C) 2005 Alexander V. Melnikoff
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/* Sorry for the long lines in the sources, I have wide screen... ;-)
 */

#include <stdafx.h>
#include <cmnfuncs.h>
#include <scripthost.h>	

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CScriptHost::CScriptHost() 
{
	cfdprintf(_T("CScriptHost::CScriptHost"));

	m_pITypeLib  = NULL;
	m_pITypeInfo = NULL;
	m_pIBBO      = NULL;
	m_pBBO       = NULL;

	cfdprintf(_T("!CScriptHost::CScriptHost"));
};

CScriptHost::~CScriptHost() 
{
	cfdprintf(_T("CScriptHost::~CScriptHost"));

	if(m_pITypeInfo) m_pITypeInfo->Release();
	if(m_pITypeLib) m_pITypeLib->Release();
	if(m_pIBBO) m_pIBBO->Release();
	if(m_pBBO) delete m_pBBO;

	cfdprintf(_T("!CScriptHost::~CScriptHost"));
};

//////////////////////////////////////////////////////////////////////
// IUnknown
//////////////////////////////////////////////////////////////////////

HRESULT __stdcall CScriptHost::QueryInterface(const IID& _riid, void** _ppv) 
{
	if(_riid == IID_IUnknown || _riid == IID_IActiveScriptSite) 
	{
		*_ppv = static_cast<IActiveScriptSite*>(this);
	} 
	else 
	{
		*_ppv = NULL;
		return E_NOINTERFACE;
	};

	reinterpret_cast<IUnknown*>(*_ppv)->AddRef ();

	return(S_OK);
};

ULONG __stdcall CScriptHost::AddRef(void) 
{
	return InterlockedIncrement(&m_cRef);
};

ULONG __stdcall CScriptHost::Release(void) 
{
	if(InterlockedDecrement(&m_cRef) == 0) 
	{
		delete this;
		return 0;
	};

	return(m_cRef);
};

//////////////////////////////////////////////////////////////////////
// IActiveScriptSite
//////////////////////////////////////////////////////////////////////

//--------------------------------------------------------------------
// Retrieves the  locale  identifier  associated with the host's  user 
// interface. The scripting engine uses the identifier to ensure  that 
// error strings and other  user-interface  elements generated  by the
// engine  appear in  the appropriate language. 
// If this method returns E_NOTIMPL, the system-defined locale 
// identifier should be used.
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::GetLCID(LCID* plcid_) // address of variable for language identifier
{	
	return(E_NOTIMPL);
};

//--------------------------------------------------------------------
// Retrieves a host-defined string that uniquely identifies the current 
// document version. If the related document has changed outside the 
// scope of Windows Script (as in the case of an HTML page being edited 
// with NotePad), the scripting engine can save this along with its 
// persisted state, forcing a recompile the next time the script is 
// loaded. 
// If E_NOTIMPL is returned, the scripting engine should assume that 
// the script is in sync with the document. 
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::GetDocVersionString(BSTR* pbsVal_) // address of document version string
{	
	return(E_NOTIMPL);
};

//--------------------------------------------------------------------
// Informs the host that the scripting engine has begun executing the 
// script code. 
// Returns S_OK if successful.
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::OnEnterScript(void) 
{
	cfdprintf(_T("CScriptHost::OnEnterScript: Event fired."));
	return(S_OK);
};

//--------------------------------------------------------------------
// Informs the host that the scripting engine has returned from 
// executing script code.
// Returns S_OK if successful.
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::OnLeaveScript(void) 
{
	cfdprintf(_T("CScriptHost::OnLeaveScript: Event fired."));
	return(S_OK);
};

//--------------------------------------------------------------------
// Informs the host that an execution error occurred while the engine 
// was running the script. 
// Returns S_OK if the error was correctly handled, or an OLE defined
// error code otherwise. 
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::OnScriptError(IActiveScriptError* pASE_) 
{	
/*
	DWORD	dwSrcCntx;
	ULONG	ulLineNum;
	LONG	lCharPos;
	TCHAR	szMsg[2048];

    EXCEPINFO epi;
	
	pASE_->GetExceptionInfo (&epi);
	pASE_->GetSourcePosition (&dwSrcCntx, &ulLineNum, &lCharPos);

	StringCchPrintf(szMsg, 2048, 
					_T("Execution of script was terminated "
					"due to an error.\r\n\tFile name: %s\r\n\tCode: %d\r\n\tSource: %s\r\n\t"
					"Description: %s\r\n\tLine Number: %d\r\n\t"
					"Char Position: %d\r\n\tSource Context: %d"), m_szFileName,
					epi.wCode, cfW2T(epi.bsSource), W2T(epi.bstrDescription), 
					ulLineNum+1, lCharPos+1, dwSrcCntx);
*/

	MessageBox(NULL, _T("Oops... Internal programm error!!!"), BBOBJ_OBJ_NAME_CAPT, MB_OK|MB_ICONERROR);

	return(S_OK);
};

//--------------------------------------------------------------------
// Informs the host that the script has completed execution.
// Returns S_OK if successful.
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::OnScriptTerminate(const VARIANT* _pvarResult,	// address of script results
												 const EXCEPINFO* _pexcepinfo)	// address of structure with exception information
{	
	cfdprintf(_T("CScriptHost::OnScriptTerminate: Event fired."));
	return(S_OK);
};

//--------------------------------------------------------------------
// Informs the host that the scripting engine has changed states.
// Returns S_OK if successful.
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::OnStateChange(SCRIPTSTATE _ssScriptState) // new state of engine
{	
	cfdprintf(_T("CScriptHost::OnStateChange: Event fired."));
	return(S_OK);
};

//--------------------------------------------------------------------
// Allows the scripting engine to obtain information about an item 
// added with the IActiveScript::AddNamedItem method.
// Returns one of the following values:
// S_OK                   - Success
// E_INVALIDARG           - An argument was invalid 
// E_POINTER              - An invalid pointer was specified 
// TYPE_E_ELEMENTNOTFOUND - An item of the specified name was not found 
//--------------------------------------------------------------------
HRESULT __stdcall CScriptHost::GetItemInfo(LPCOLESTR _pstrName,			// address of item name
										   DWORD _dwReturnMask,			// bit mask for information retrieval
										   IUnknown** _ppunkItem,		// address of pointer to item's IUnknown
										   ITypeInfo** _ppTypeInfo)		// address of pointer to item's ITypeInfo
{	
	if(_ppTypeInfo) 
	{
		*_ppTypeInfo = NULL;
		// If we was asked for ITypeInfo.
		if(_dwReturnMask & SCRIPTINFO_ITYPEINFO) 
		{
			m_pITypeInfo->AddRef();
			*_ppTypeInfo = m_pITypeInfo;
		};
	};

	// If Windows Scripting placed _ppunkItem...
	if(_ppunkItem) 
	{
		*_ppunkItem = NULL;
		// If we were asked for IUnknown.
		if(_dwReturnMask & SCRIPTINFO_IUNKNOWN) 
		{
//			cfdprintf(_T("CScriptHost::GetItemInfo: Obj: %s"), cfW2T((LPWSTR)_pstrName));
			if(!_tcscmp(_T("BotikBalance"), CW2T(_pstrName)))  // We was asked for HostObject object.
			{
				m_pIBBO->AddRef();
				*_ppunkItem = m_pIBBO;
			};
		};
	};	// if(_ppunkItem)

	return(S_OK);
};

//////////////////////////////////////////////////////////////////////
// CScriptHost
//////////////////////////////////////////////////////////////////////

CScriptHost* CScriptHost::CreateObject()
{
	CScriptHost* pObj = NULL;
	pObj = new CScriptHost();
	return(pObj);
};

BOOL CScriptHost::Init(VOID) 
{
	USES_CONVERSION;

	BOOL	fOk = FALSE;

	cfdprintf(_T("CScriptHost::Init"));

	__try
	{
		TCHAR	szPath[MAX_PATH+2+1]; // 2 is for resource id and slash, see below...
		HRESULT	hr;

		// At this step we need to load type library. The type library
		// for this object is located in this DLL resource section.
		// To load type library from this DLL, we need to know where 
		// this DLL is located. After getting DLL path, we need to 
		// add resource number of interested type library at the end of path.
		// If we do not do that, we will load first type library. First 
		// type library is for COM/ActiveX interface.
		GetModuleFileName((HMODULE)g_hModule, szPath, MAX_PATH);
		_tcscat(szPath, _T("\\2"));

		cfdprintf(_T("CScriptHost::Init: Loading type library from %s"), szPath);

		hr = ::LoadTypeLib(T2W(szPath), &m_pITypeLib);

		if(FAILED(hr)) 
		{
			cfdprintf(_T("CScriptHost::Init: Load type library failed (0x%x)"), hr);
			__leave;
		};

		// Get type information for interface object.
		hr = m_pITypeLib->GetTypeInfoOfGuid(CLSID_BotikBalanceObj, &m_pITypeInfo);
		if(FAILED(hr)) 
		{
			cfdprintf(_T("CScriptHost::Init: Get type information for object object failed (0x%x)"), hr);
			__leave;
		};

		// Init object for script scope.
		m_pBBO = CBotikBalanceObj::CreateObject();
		if(m_pBBO == NULL)
		{
			cfdprintf(_T("CScriptHost::Init: Create CBotikBalanceObj failed"));
			__leave;
		};

		m_pITypeLib->AddRef();
		if(!m_pBBO->Init(m_pITypeLib))
		{
			cfdprintf(_T("CScriptHost::Init: Create CBotikBalanceObj::Init failed"));
			__leave;
		};

		hr = m_pBBO->QueryInterface(IID_IUnknown, (void**)&m_pIBBO);
		if(FAILED(hr)) 
		{
			cfdprintf(_T("CScriptHost::Init: Create CBotikBalanceObj::QueryInterface failed"));
			__leave;
		};

		fOk = TRUE;

		cfdprintf(_T("CScriptHost::Init: Success"));
	}

	__finally
	{
		cfdprintf(_T("CScriptHost::Init: __finaly"));
	};

	cfdprintf(_T("!CScriptHost::Init"));

	return(fOk);
};




